@@ -46,6 +46,7 @@
 #include <audacious/i18n.h>
 
 #include <audacious/plugin.h>
+#include <audacious/main.h>
 #include <audacious/output.h>
 #include <audacious/vfs.h>
 #include <audacious/util.h>
@@ -63,27 +64,16 @@ static int seek_to = -1;
 
 extern void set_endian();
 
-static gchar *
-extname(const char *filename)
-{
-    gchar *ext = strrchr(filename, '.');
-
-    if (ext != NULL)
-        ++ext;
-
-    return ext;  
-}
-
 static void alac_about(void)
 {
 	static GtkWidget *aboutbox;
-    
+
 	if(aboutbox != NULL)
 		return;
 
-	aboutbox = xmms_show_message("About Apple Lossless Audio Plugin",
-				     "Copyright (c) 2006 Audacious team\n"
-				     "Portions (c) 2005-2006 David Hammerton <crazney -at- crazney.net>",
+	aboutbox = audacious_info_dialog(_("About Apple Lossless Audio Plugin"),
+				     _("Copyright (c) 2006 Audacious team\n"
+				     "Portions (c) 2005-2006 David Hammerton <crazney -at- crazney.net>"),
 				     _("Ok"), FALSE, NULL, NULL);
 
 	g_signal_connect(G_OBJECT(aboutbox), "destroy",
@@ -91,11 +81,6 @@ static void alac_about(void)
 			 &aboutbox);
 }
 
-static void alac_init(void)
-{
-	/* empty */
-}
-
 gboolean is_our_fd(char *filename, VFSFile* input_file)
 {
     demux_res_t demux_res;
@@ -107,7 +92,7 @@ gboolean is_our_fd(char *filename, VFSFile* input_file)
 
     if (!input_stream)
         return FALSE;
-        
+
     /* if qtmovie_read returns successfully, the stream is up to
      * the movie data, which can be used directly by the decoder */
     if (!qtmovie_read(input_stream, &demux_res))
@@ -121,61 +106,59 @@ gboolean is_our_fd(char *filename, VFSFile* input_file)
     return TRUE;
 }
 
-TitleInput *build_tuple_from_demux(demux_res_t *demux_res, char *path)
+Tuple *build_aud_tuple_from_demux(demux_res_t *demux_res, char *path)
 {
-    TitleInput *ti = bmp_title_input_new();
+    Tuple *ti = aud_tuple_new_from_filename(path);
 
     if (demux_res->tuple.art != NULL)
-        ti->performer = g_strdup(demux_res->tuple.art);
+        aud_tuple_associate_string(ti, FIELD_ARTIST, NULL, demux_res->tuple.art);
     if (demux_res->tuple.nam != NULL)
-        ti->track_name = g_strdup(demux_res->tuple.nam);
+        aud_tuple_associate_string(ti, FIELD_TITLE, NULL, demux_res->tuple.nam);
     if (demux_res->tuple.alb != NULL)
-        ti->album_name = g_strdup(demux_res->tuple.alb);
+        aud_tuple_associate_string(ti, FIELD_ALBUM, NULL, demux_res->tuple.alb);
     if (demux_res->tuple.gen != NULL)
-        ti->genre = g_strdup(demux_res->tuple.gen);
+        aud_tuple_associate_string(ti, FIELD_GENRE, NULL, demux_res->tuple.gen);
     if (demux_res->tuple.cmt != NULL)
-        ti->comment = g_strdup(demux_res->tuple.cmt);
+        aud_tuple_associate_string(ti, FIELD_COMMENT, NULL, demux_res->tuple.cmt);
     if (demux_res->tuple.day != NULL)
-        ti->year = atoi(demux_res->tuple.day);
+        aud_tuple_associate_int(ti, FIELD_YEAR, NULL, atoi(demux_res->tuple.day));
 
-    ti->file_name = g_path_get_basename(path);
-    ti->file_path = g_path_get_dirname(path);
-    ti->file_ext = extname(path);
+    aud_tuple_associate_string(ti, FIELD_CODEC, NULL, "Apple Lossless (ALAC)");
+    aud_tuple_associate_string(ti, FIELD_QUALITY, NULL, "lossless");
 
     return ti;
 }
 
-TitleInput *build_tuple(char *filename)
+Tuple *build_tuple(char *filename)
 {
     demux_res_t demux_res;
     VFSFile *input_file;
     stream_t *input_stream;
-    TitleInput *ti;
 
-    input_file = vfs_fopen(filename, "rb");
+    input_file = aud_vfs_fopen(filename, "rb");
     input_stream = stream_create_file(input_file, 1);
 
     set_endian();
 
     if (!input_stream)
     {
-	vfs_fclose(input_file);
+	aud_vfs_fclose(input_file);
         return NULL;
     }
-        
+
     /* if qtmovie_read returns successfully, the stream is up to
      * the movie data, which can be used directly by the decoder */
     if (!qtmovie_read(input_stream, &demux_res))
     {
         stream_destroy(input_stream);
-	vfs_fclose(input_file);
+        aud_vfs_fclose(input_file);
         return NULL;
     }
 
     stream_destroy(input_stream);
-    vfs_fclose(input_file);
+    aud_vfs_fclose(input_file);
 
-    return build_tuple_from_demux(&demux_res, filename);
+    return build_aud_tuple_from_demux(&demux_res, filename);
 }
 
 static InputPlayback *playback;
@@ -185,19 +168,21 @@ static void play_file(InputPlayback *data)
     char *filename = data->filename;
     going = 1;
     playback = data;
-    playback_thread = g_thread_create(decode_thread, filename, TRUE, NULL);
+    playback_thread = g_thread_self();
+    playback->set_pb_ready(playback);
+    decode_thread(filename);
 }
 
 static void stop(InputPlayback * data)
 {
     going = 0;
     g_thread_join(playback_thread);
-    output_close_audio();
+    data->output->close_audio();
 }
 
 static void do_pause(InputPlayback *data, short paused)
 {
-    output_pause(paused);
+    data->output->pause(paused);
 }
 
 static void seek(InputPlayback * data, gint time)
@@ -205,48 +190,6 @@ static void seek(InputPlayback * data, gint time)
     seek_to = time;
 }
 
-static gint get_time(InputPlayback *data)
-{
-    if (going)
-	return get_output_time();
-    else
-	return -1;
-}
-
-gchar *alac_fmts[] = { "m4a", "alac", NULL };
-
-InputPlugin alac_ip = {
-    NULL,
-    NULL,
-    "Apple Lossless Plugin",                       /* Description */  
-    alac_init,
-    alac_about,
-    NULL,
-    NULL,
-    NULL,
-    play_file,
-    stop,
-    do_pause,
-    seek,
-    NULL,
-    get_time,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,                       /* file_info_box */
-    NULL,
-    build_tuple,
-    NULL,
-    NULL,
-    is_our_fd,
-    alac_fmts
-};
-
 static int get_sample_info(demux_res_t *demux_res, uint32_t samplenum,
                            uint32_t *sample_duration,
                            uint32_t *sample_byte_size)
@@ -329,29 +272,45 @@ void GetBuffer(demux_res_t *demux_res)
         /* write */
         bytes_read += outputBytes;
 
-        produce_audio(playback->output->written_time(), FMT_S16_LE, demux_res->num_channels, outputBytes, pDestBuffer, &going);
+        playback->pass_audio(playback, FMT_S16_LE, demux_res->num_channels, outputBytes, pDestBuffer, &going);
     }
 
     free(buffer);
     free(pDestBuffer);
 }
 
+static gchar *fmts[] = { "m4a", "alac", NULL };
+
+InputPlugin alac_ip = {
+    .description = "Apple Lossless Plugin",
+    .about = alac_about,
+    .play_file = play_file,
+    .stop = stop,
+    .pause = do_pause,
+    .seek = seek,
+    .get_song_tuple = build_tuple,
+    .is_our_file_from_vfs = is_our_fd,
+    .vfs_extensions = fmts,
+};
+
+InputPlugin *alac_iplist[] = { &alac_ip, NULL };
+
+DECLARE_PLUGIN(alac, NULL, NULL, alac_iplist, NULL, NULL, NULL, NULL, NULL);
+
 gpointer decode_thread(void *args)
 {
     demux_res_t demux_res;
-    unsigned int output_size, i;
     gulong duration = 0;	/* samples added up */
-    gint framesize;
     VFSFile *input_file;
     stream_t *input_stream;
-    TitleInput *ti;
+    Tuple *ti;
     gchar *title;
 
     memset(&demux_res, 0, sizeof(demux_res));
 
     set_endian();
 
-    input_file = vfs_fopen((char *) args, "rb");
+    input_file = aud_vfs_fopen((char *) args, "rb");
     input_stream = stream_create_file(input_file, 1);
 
     if (!input_stream)
@@ -365,8 +324,8 @@ gpointer decode_thread(void *args)
     demux_res.stream = input_stream;
 
     /* Get the titlestring ready. */
-    ti = build_tuple_from_demux(&demux_res, (char *) args);
-    title = xmms_get_titlestring(xmms_get_gentitle_format(), ti);
+    ti = build_aud_tuple_from_demux(&demux_res, (char *) args);
+    title = aud_tuple_formatter_make_title_string(ti, aud_get_gentitle_format());
 
     /* initialise the sound converter */
     demux_res.alac = create_alac(demux_res.sample_size, demux_res.num_channels);
@@ -377,7 +336,7 @@ gpointer decode_thread(void *args)
 	(float)(demux_res.sample_rate / 251));
 
     playback->output->open_audio(FMT_S16_LE, demux_res.sample_rate, demux_res.num_channels);
-    alac_ip.set_info(title, duration, -1, demux_res.sample_rate, demux_res.num_channels);
+    playback->set_params(playback, title, duration, -1, demux_res.sample_rate, demux_res.num_channels);
 
     /* will convert the entire buffer */
     GetBuffer(&demux_res);
@@ -387,14 +346,9 @@ gpointer decode_thread(void *args)
     stream_destroy(input_stream);
 
     if (input_opened)
-        vfs_fclose(input_file);
+        aud_vfs_fclose(input_file);
 
     playback->output->close_audio();
 
     return NULL;
 }
-
-InputPlugin *get_iplugin_info(void)
-{
-    return &alac_ip;
-}
