@@ -32,7 +32,7 @@ gint backend_info_get( gchar ** name , gchar ** longname , gchar ** desc , gint
   if ( name != NULL )
     *name = g_strdup( "alsa" );
   if ( longname != NULL )
-    *longname = g_strdup( "ALSA Backend " AMIDIPLUG_VERSION );
+    *longname = g_strjoin( "", _("ALSA Backend "), AMIDIPLUG_VERSION, NULL );
   if ( desc != NULL )
     *desc = g_strdup( _("This backend sends MIDI events to a group of user-chosen "
                         "ALSA sequencer ports. The ALSA sequencer interface is very "
@@ -179,6 +179,12 @@ gint sequencer_queue_start( void )
 }
 
 
+gint sequencer_queue_stop( void )
+{
+  return snd_seq_stop_queue( sc.seq , sc.queue , NULL );
+}
+
+
 gint sequencer_event_init( void )
 {
   /* common settings for all our events */
@@ -291,6 +297,31 @@ gint sequencer_event_other( midievent_t * event )
 }
 
 
+gint sequencer_event_allnoteoff( gint unused )
+{
+  gint i = 0 , c = 0;
+  /* send "ALL SOUNDS OFF" to all channels on all ports */
+  sc.ev.type = SND_SEQ_EVENT_CONTROLLER;
+  sc.ev.time.tick = 0;
+  snd_seq_ev_set_fixed(&sc.ev);
+  sc.ev.data.control.param = MIDI_CTL_ALL_SOUNDS_OFF;
+  sc.ev.data.control.value = 0;
+  for ( i = 0 ; i < sc.dest_port_num ; i++ )
+  {
+    sc.ev.queue = sc.queue;
+    sc.ev.dest = sc.dest_port[i];
+
+    for ( c = 0 ; c < 16 ; c++ )
+    {
+      sc.ev.data.control.channel = c;
+      snd_seq_event_output(sc.seq, &sc.ev);
+      snd_seq_drain_output(sc.seq);
+    }
+  }
+  return 1;
+}
+
+
 gint sequencer_output( gpointer * buffer , gint * len )
 {
   snd_seq_event_output( sc.seq , &sc.ev );
