@@ -1,6 +1,6 @@
 /*
  * mad plugin for audacious
- * Copyright (C) 2005-2007 William Pitcock, Yoshiki Yazawa
+ * Copyright (C) 2005-2007 William Pitcock, Yoshiki Yazawa, Eugene Zagidullin
  *
  * Portions derived from xmms-mad:
  * Copyright (C) 2001-2002 Sam Clegg - See COPYING
@@ -19,6 +19,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/* #define AUD_DEBUG 1 */
+
 #include "config.h"
 #include "plugin.h"
 #include "input.h"
@@ -32,12 +34,12 @@
 #include <fcntl.h>
 #include <audacious/vfs.h>
 #include <sys/stat.h>
-#include "SFMT.h"
+#include "tuple.h"
 
 /*
  * Global variables
  */
-struct audmad_config_t audmad_config;   /**< global configuration */
+audmad_config_t *audmad_config;   /**< global configuration */
 GMutex *mad_mutex;
 GMutex *pb_mutex;
 GCond *mad_cond;
@@ -76,7 +78,8 @@ static gint mp3_samplerate_table[4][4] = {
  *    filename has no extension.
  *
  */
-static gchar *extname(const char *filename)
+static gchar *
+extname(const char *filename)
 {
     gchar *ext = strrchr(filename, '.');
 
@@ -87,76 +90,45 @@ static gchar *extname(const char *filename)
 }
 
 
-void audmad_config_compute(struct audmad_config_t *config)
-{
-    /* set some config parameters by parsing text fields
-       (RG default gain, etc..)
-     */
-    const gchar *text;
-    gdouble x;
-
-    text = config->pregain_db;
-    if ( text != NULL )
-      x = g_strtod(text, NULL);
-    else
-      x = 0;
-    config->pregain_scale = (x != 0) ? pow(10.0, x / 20) : 1;
-#ifdef DEBUG
-    g_message("pregain=[%s] -> %g  -> %g", text, x, config->pregain_scale);
-#endif
-    text = config->replaygain.default_db;
-    if ( text != NULL )
-      x = g_strtod(text, NULL);
-    else
-      x = 0;
-    config->replaygain.default_scale = (x != 0) ? pow(10.0, x / 20) : 1;
-#ifdef DEBUG
-    g_message("RG.default=[%s] -> %g  -> %g", text, x,
-              config->replaygain.default_scale);
-#endif
-}
-
-static void audmad_init()
+static void
+audmad_init()
 {
     ConfigDb *db = NULL;
 
-    audmad_config.fast_play_time_calc = TRUE;
-    audmad_config.use_xing = TRUE;
-    audmad_config.dither = TRUE;
-    audmad_config.sjis = FALSE;
-    audmad_config.hard_limit = FALSE;
-    audmad_config.replaygain.enable = TRUE;
-    audmad_config.replaygain.track_mode = FALSE;
-    audmad_config.title_override = FALSE;
-    audmad_config.show_avg_vbr_bitrate = TRUE;
-    audmad_config.force_reopen_audio = FALSE;
+    audmad_config = g_malloc0(sizeof(audmad_config_t));
+
+    audmad_config->dither = TRUE;
+    audmad_config->force_reopen_audio = FALSE;
+    audmad_config->fast_play_time_calc = TRUE;
+    audmad_config->use_xing = TRUE;
+    audmad_config->sjis = FALSE;
+    audmad_config->show_avg_vbr_bitrate = TRUE;
+    audmad_config->title_override = FALSE;
+
 
     db = aud_cfg_db_open();
     if (db) {
-        aud_cfg_db_get_bool(db, "MAD", "fast_play_time_calc",
-                            &audmad_config.fast_play_time_calc);
-        aud_cfg_db_get_bool(db, "MAD", "use_xing",
-                            &audmad_config.use_xing);
-        aud_cfg_db_get_bool(db, "MAD", "dither", &audmad_config.dither);
-        aud_cfg_db_get_bool(db, "MAD", "sjis", &audmad_config.sjis);
-        aud_cfg_db_get_bool(db, "MAD", "hard_limit",
-                            &audmad_config.hard_limit);
-        aud_cfg_db_get_string(db, "MAD", "pregain_db",
-                              &audmad_config.pregain_db);
-        aud_cfg_db_get_bool(db, "MAD", "RG.enable",
-                            &audmad_config.replaygain.enable);
-        aud_cfg_db_get_bool(db, "MAD", "RG.track_mode",
-                            &audmad_config.replaygain.track_mode);
-        aud_cfg_db_get_string(db, "MAD", "RG.default_db",
-                              &audmad_config.replaygain.default_db);
-        aud_cfg_db_get_bool(db, "MAD", "title_override",
-                            &audmad_config.title_override);
-        aud_cfg_db_get_string(db, "MAD", "id3_format",
-                              &audmad_config.id3_format);
-        aud_cfg_db_get_bool(db, "MAD", "show_avg_vbr_bitrate",
-                            &audmad_config.show_avg_vbr_bitrate);
+        //audio
+        aud_cfg_db_get_bool(db, "MAD", "dither", &audmad_config->dither);
         aud_cfg_db_get_bool(db, "MAD", "force_reopen_audio",
-                            &audmad_config.force_reopen_audio);
+                            &audmad_config->force_reopen_audio);
+
+        //metadata
+        aud_cfg_db_get_bool(db, "MAD", "fast_play_time_calc",
+                            &audmad_config->fast_play_time_calc);
+        aud_cfg_db_get_bool(db, "MAD", "use_xing",
+                            &audmad_config->use_xing);
+        aud_cfg_db_get_bool(db, "MAD", "sjis", &audmad_config->sjis);
+
+        //misc
+        aud_cfg_db_get_bool(db, "MAD", "show_avg_vbr_bitrate",
+                            &audmad_config->show_avg_vbr_bitrate);
+
+        //text
+        aud_cfg_db_get_bool(db, "MAD", "title_override",
+                            &audmad_config->title_override);
+        aud_cfg_db_get_string(db, "MAD", "id3_format",
+                              &audmad_config->id3_format);
 
         aud_cfg_db_close(db);
     }
@@ -164,38 +136,26 @@ static void audmad_init()
     mad_mutex = g_mutex_new();
     pb_mutex = g_mutex_new();
     mad_cond = g_cond_new();
-    audmad_config_compute(&audmad_config);
 
-    if (!audmad_config.pregain_db)
-        audmad_config.pregain_db = g_strdup("+0.00");
-
-    if (!audmad_config.replaygain.default_db)
-        audmad_config.replaygain.default_db = g_strdup("-9.00");
-
-    if (!audmad_config.id3_format)
-        audmad_config.id3_format = g_strdup("");
-
-    init_gen_rand(4357);
+    if (!audmad_config->id3_format)
+        audmad_config->id3_format = g_strdup("(none)");
 
     aud_mime_set_plugin("audio/mpeg", mad_plugin);
 }
 
-static void audmad_cleanup()
+static void
+audmad_cleanup()
 {
-    g_free(audmad_config.pregain_db);
-    g_free(audmad_config.replaygain.default_db);
-    g_free(audmad_config.id3_format);
-
-    audmad_config.pregain_db = NULL;
-    audmad_config.replaygain.default_db = NULL;
-    audmad_config.id3_format = NULL;
-
+    g_free(audmad_config->id3_format);
+    g_free(audmad_config);
+    
     g_cond_free(mad_cond);
     g_mutex_free(mad_mutex);
     g_mutex_free(pb_mutex);
 }
 
-static gboolean mp3_head_check(guint32 head, gint *frameSize)
+static gboolean
+mp3_head_check(guint32 head, gint *frameSize)
 {
     gint version, layer, bitIndex, bitRate, sampleIndex, sampleRate, padding;
 
@@ -283,21 +243,17 @@ static gboolean mp3_head_check(guint32 head, gint *frameSize)
     return TRUE;
 }
 
-static int mp3_head_convert(const guchar * hbuf)
+static int
+mp3_head_convert(const guchar * hbuf)
 {
     return ((unsigned long) hbuf[0] << 24) |
         ((unsigned long) hbuf[1] << 16) |
         ((unsigned long) hbuf[2] << 8) | (unsigned long) hbuf[3];
 }
 
-gboolean audmad_is_remote(gchar *url)
-{
-    gboolean rv = aud_vfs_is_remote(url);
-    return rv;
-}
-
 // audacious vfs fast version
-static int audmad_is_our_fd(char *filename, VFSFile *fin)
+static int
+audmad_is_our_fd(char *filename, VFSFile *fin)
 {
     guint32 check;
     gchar *ext = extname(filename);
@@ -306,10 +262,7 @@ static int audmad_is_our_fd(char *filename, VFSFile *fin)
     guchar tmp[4096];
     gint ret, i, frameSize;
 
-    info.remote = FALSE;
-
-    if(audmad_is_remote(filename))
-        info.remote = TRUE;
+    info.remote = aud_vfs_is_remote(filename);
 
     /* I've seen some flac files beginning with id3 frames..
        so let's exclude known non-mp3 filename extensions */
@@ -386,7 +339,8 @@ static int audmad_is_our_fd(char *filename, VFSFile *fin)
 }
 
 // audacious vfs version
-static int audmad_is_our_file(char *filename)
+static int
+audmad_is_our_file(char *filename)
 {
     VFSFile *fin = NULL;
     gint rtn;
@@ -402,11 +356,10 @@ static int audmad_is_our_file(char *filename)
     return rtn;
 }
 
-static void audmad_stop(InputPlayback *playback)
+static void
+audmad_stop(InputPlayback *playback)
 {
-#ifdef DEBUG
-    g_message("f: audmad_stop");
-#endif
+    AUDDBG("f: audmad_stop\n");
     g_mutex_lock(mad_mutex);
     info.playback = playback;
     g_mutex_unlock(mad_mutex);
@@ -418,31 +371,28 @@ static void audmad_stop(InputPlayback *playback)
         g_mutex_unlock(mad_mutex);
         g_cond_signal(mad_cond);
 
-#ifdef DEBUG
-        g_message("waiting for thread");
-#endif
+        AUDDBG("waiting for thread\n");
         g_thread_join(decode_thread);
-#ifdef DEBUG
-        g_message("thread done");
-#endif
+        AUDDBG("thread done\n");
+
         input_term(&info);
         decode_thread = NULL;
 
     }
-#ifdef DEBUG
-    g_message("e: audmad_stop");
-#endif
+    AUDDBG("e: audmad_stop\n");
 }
 
-static void audmad_play_file(InputPlayback *playback)
+static void
+audmad_play_file(InputPlayback *playback)
 {
     gboolean rtn;
     gchar *url = playback->filename;
+    ReplayGainInfo rg_info;
 
-#ifdef DEBUG
+#ifdef AUD_DEBUG
     {
         gchar *tmp = g_filename_to_utf8(url, -1, NULL, NULL, NULL);
-        g_message("playing %s", tmp);
+        AUDDBG("playing %s\n", tmp);
         g_free(tmp);
     }
 #endif                          /* DEBUG */
@@ -453,7 +403,7 @@ static void audmad_play_file(InputPlayback *playback)
     }
 
     // remote access must use fast scan.
-    rtn = input_get_info(&info, audmad_is_remote(url) ? TRUE : audmad_config.fast_play_time_calc);
+    rtn = input_get_info(&info, aud_vfs_is_remote(url) ? TRUE : audmad_config->fast_play_time_calc);
 
     if (rtn == FALSE) {
         g_message("error reading input info");
@@ -463,6 +413,18 @@ static void audmad_play_file(InputPlayback *playback)
          * that used to work only for nenolod because of his fsck-ing lastfm subscription :p
         */
     }
+
+    rg_info.track_gain = info.replaygain_track_scale;
+    rg_info.track_peak = info.replaygain_track_peak;
+    rg_info.album_gain = info.replaygain_album_scale;
+    rg_info.album_peak = info.replaygain_album_peak;
+    AUDDBG("Replay Gain info:\n");
+    AUDDBG("* track gain:          %+f dB\n", rg_info.track_gain);
+    AUDDBG("* track peak:          %f\n",     rg_info.track_peak);
+    AUDDBG("* album gain:          %+f dB\n", rg_info.album_gain);
+    AUDDBG("* album peak:          %f\n",     rg_info.album_peak);
+    playback->set_replaygain_info(playback, &rg_info);
+
     g_mutex_lock(pb_mutex);
     info.playback = playback;
     info.playback->playing = 1;
@@ -473,7 +435,8 @@ static void audmad_play_file(InputPlayback *playback)
     decode_loop(&info);
 }
 
-static void audmad_pause(InputPlayback *playback, short paused)
+static void
+audmad_pause(InputPlayback *playback, short paused)
 {
     g_mutex_lock(pb_mutex);
     info.playback = playback;
@@ -481,7 +444,8 @@ static void audmad_pause(InputPlayback *playback, short paused)
     playback->output->pause(paused);
 }
 
-static void audmad_mseek(InputPlayback *playback, gulong millisecond)
+static void
+audmad_mseek(InputPlayback *playback, gulong millisecond)
 {
     g_mutex_lock(pb_mutex);
     info.playback = playback;
@@ -489,7 +453,8 @@ static void audmad_mseek(InputPlayback *playback, gulong millisecond)
     g_mutex_unlock(pb_mutex);
 }
 
-static void audmad_seek(InputPlayback *playback, gint time)
+static void
+audmad_seek(InputPlayback *playback, gint time)
 {
     audmad_mseek(playback, time * 1000);
 }
@@ -502,20 +467,18 @@ static void
 audmad_get_song_info(char *url, char **title, int *length)
 {
     struct mad_info_t myinfo;
-#ifdef DEBUG
+#ifdef AUD_DEBUG
     gchar *tmp = g_filename_to_utf8(url, -1, NULL, NULL, NULL);
-    g_message("f: audmad_get_song_info: %s", tmp);
+    AUDDBG("f: audmad_get_song_info: %s\n", tmp);
     g_free(tmp);
 #endif                          /* DEBUG */
 
     if (input_init(&myinfo, url, NULL) == FALSE) {
-#ifdef DEBUG
-        g_message("error initialising input");
-#endif
+        AUDDBG("error initialising input\n");
         return;
     }
 
-    if (input_get_info(&myinfo, info.remote ? TRUE : audmad_config.fast_play_time_calc) == TRUE) {
+    if (input_get_info(&myinfo, info.remote ? TRUE : audmad_config->fast_play_time_calc) == TRUE) {
         if(aud_tuple_get_string(myinfo.tuple, -1, "track-name"))
             *title = g_strdup(aud_tuple_get_string(myinfo.tuple, -1, "track-name"));
         else
@@ -530,43 +493,26 @@ audmad_get_song_info(char *url, char **title, int *length)
         *length = -1;
     }
     input_term(&myinfo);
-#ifdef DEBUG
-    g_message("e: audmad_get_song_info");
-#endif                          /* DEBUG */
+    AUDDBG("e: audmad_get_song_info\n");
+}
+
+static gboolean
+audmad_fill_info(struct mad_info_t *info, VFSFile *fd)
+{
+    if (fd == NULL || info == NULL) return FALSE;
+    AUDDBG("f: audmad_fill_info(): %s\n", fd->uri);
+
+    if (input_init(info, fd->uri, fd) == FALSE) {
+        AUDDBG("audmad_fill_info(): error initialising input\n");
+        return FALSE;
+    }
+    
+    info->fileinfo_request = FALSE; /* we don't need to read tuple again */
+    return input_get_info(info, aud_vfs_is_remote(fd->uri) ? TRUE : audmad_config->fast_play_time_calc);
 }
 
 static void
-audmad_get_song_length(char *url, int *length, VFSFile *fd)
-{
-    struct mad_info_t myinfo;
-#ifdef DEBUG
-    gchar *tmp = g_filename_to_utf8(url, -1, NULL, NULL, NULL);
-    g_message("f: audmad_get_song_length: %s", tmp);
-    g_free(tmp);
-#endif                          /* DEBUG */
-
-    if (input_init(&myinfo, url, fd ? fd : NULL) == FALSE) {
-#ifdef DEBUG
-        g_message("error initialising input");
-#endif
-        return;
-    }
-
-    if (input_get_info(&myinfo, info.remote ? TRUE : audmad_config.fast_play_time_calc) == TRUE) {
-        *length = aud_tuple_get_int(myinfo.tuple, FIELD_LENGTH, NULL);
-        if(*length == -1)
-            *length = mad_timer_count(myinfo.duration, MAD_UNITS_MILLISECONDS);
-    }
-    else {
-        *length = -1;
-    }
-    input_term(&myinfo);
-#ifdef DEBUG
-    g_message("e: audmad_get_song_info");
-#endif                          /* DEBUG */
-}
-
-static void audmad_about()
+audmad_about()
 {
     static GtkWidget *aboutbox;
     gchar *scratch;
@@ -605,7 +551,8 @@ static void audmad_about()
  * Display a GTK box containing the given error message.
  * Taken from mpg123 plugin.
  */
-void audmad_error(char *error, ...)
+void
+audmad_error(char *error, ...)
 {
 #ifndef NOGUI
     if (!error_dialog) {
@@ -625,42 +572,47 @@ void audmad_error(char *error, ...)
 #endif                          /* !NOGUI */
 }
 
-extern void audmad_get_file_info(char *filename);
 extern void audmad_configure();
 
-static void __set_and_free(Tuple *tuple, gint nfield, gchar *name, gchar *value)
+static void
+__set_and_free(Tuple *tuple, gint nfield, gchar *name, gchar *value)
 {
     aud_tuple_associate_string(tuple, nfield, name, value);
     g_free(value);
 }
 
 // tuple stuff
-static Tuple *__audmad_get_song_tuple(char *filename, VFSFile *fd)
+static Tuple *
+__audmad_get_song_tuple(char *filename, VFSFile *fd)
 {
     Tuple *tuple = NULL;
     gchar *string = NULL;
-    gchar *realfn = NULL;
 
     struct id3_file *id3file = NULL;
     struct id3_tag *tag = NULL;
 
-    gboolean local_fd = FALSE;
+    struct mad_info_t myinfo;
 
-#ifdef DEBUG
+    gboolean local_fd = FALSE;
+    int length;
+
+#ifdef AUD_DEBUG
     string = aud_str_to_utf8(filename);
-    g_message("f: mad: audmad_get_song_tuple: %s", string);
+    AUDDBG("f: mad: audmad_get_song_tuple: %s\n", string);
     g_free(string);
     string = NULL;
 #endif
 
+    /* isn't is obfuscated? --eugene */
+
     if(info.remote && mad_timer_count(info.duration, MAD_UNITS_SECONDS) <= 0){
         if((fd && aud_vfs_is_streaming(fd)) || (info.playback && info.playback->playing)) {
             gchar *tmp = NULL;
             tuple = aud_tuple_new_from_filename(filename);
 
-#ifdef DEBUG
+#ifdef AUD_DEBUG
             if(info.playback)
-                g_message("info.playback->playing = %d",info.playback->playing);
+                AUDDBG("info.playback->playing = %d\n",info.playback->playing);
 #endif
             tmp = aud_vfs_get_metadata(info.infile ? info.infile : fd, "track-name");
             if(tmp){
@@ -685,21 +637,14 @@ static Tuple *__audmad_get_song_tuple(char *filename, VFSFile *fd)
                 tmp = NULL;
             }
 
-#ifdef DEBUG
-            g_message("audmad_get_song_tuple: track_name = %s", aud_tuple_get_string(tuple, -1, "track-name"));
-            g_message("audmad_get_song_tuple: stream_name = %s", aud_tuple_get_string(tuple, -1, "stream-name"));
-#endif
+            AUDDBG("audmad_get_song_tuple: track_name = %s\n", aud_tuple_get_string(tuple, -1, "track-name"));
+            AUDDBG("audmad_get_song_tuple: stream_name = %s\n", aud_tuple_get_string(tuple, -1, "stream-name"));
             aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, -1);
             aud_tuple_associate_int(tuple, FIELD_MTIME, NULL, 0); // this indicates streaming
-#ifdef DEBUG
-            g_message("get_song_tuple: remote: tuple");
-#endif
+            AUDDBG("get_song_tuple: remote: tuple\n");
             return tuple;
         }
-#ifdef DEBUG
-        g_message("get_song_tuple: remote: NULL");
-#endif
-//        return NULL;
+        AUDDBG("get_song_tuple: remote: NULL\n");
     } /* info.remote  */
 
     // if !fd, pre-open the file with aud_vfs_fopen() and reuse fd.
@@ -710,6 +655,12 @@ static Tuple *__audmad_get_song_tuple(char *filename, VFSFile *fd)
         local_fd = TRUE;
     }
 
+    if (!audmad_fill_info(&myinfo, fd)) {
+        AUDDBG("get_song_tuple: error obtaining info\n");
+        if (local_fd) aud_vfs_fclose(fd);
+        return NULL;
+    }
+
     tuple = aud_tuple_new();
     aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, -1);
 
@@ -734,29 +685,14 @@ static Tuple *__audmad_get_song_tuple(char *filename, VFSFile *fd)
                 g_free(string);
                 string = NULL;
             }
-            realfn = g_filename_from_uri(filename, NULL, NULL);
-            __set_and_free(tuple, FIELD_FILE_NAME, NULL, g_path_get_basename(realfn ? realfn : filename));
-            __set_and_free(tuple, FIELD_FILE_PATH, NULL, g_path_get_dirname(realfn ? realfn : filename));
-            aud_tuple_associate_string(tuple, FIELD_FILE_EXT, NULL, extname(realfn ? realfn : filename));
-            g_free(realfn); realfn = NULL;
+
+            __set_and_free(tuple, FIELD_FILE_NAME, NULL, aud_uri_to_display_basename(filename));
+            __set_and_free(tuple, FIELD_FILE_PATH, NULL, aud_uri_to_display_dirname(filename));
+            aud_tuple_associate_string(tuple, FIELD_FILE_EXT, NULL, extname(filename));
 
             // length
-            string = input_id3_get_string(tag, "TLEN");
-            if (string) {
-                aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, atoi(string));
-#ifdef DEBUG
-                g_message("get_song_tuple: TLEN = %d", aud_tuple_get_int(tuple, FIELD_LENGTH, NULL));
-#endif
-                g_free(string);
-                string = NULL;
-            }
-            else {
-                char *dummy = NULL;
-                int length = 0;
-                audmad_get_song_length(filename, &length, fd);
-                aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, length);
-                g_free(dummy);
-            }
+            length = mad_timer_count(myinfo.duration, MAD_UNITS_MILLISECONDS);
+            aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, length);
 
             // track number
             string = input_id3_get_string(tag, ID3_FRAME_TRACK);
@@ -768,48 +704,45 @@ static Tuple *__audmad_get_song_tuple(char *filename, VFSFile *fd)
             // genre
             __set_and_free(tuple, FIELD_GENRE, NULL, input_id3_get_string(tag, ID3_FRAME_GENRE));
             __set_and_free(tuple, FIELD_COMMENT, NULL, input_id3_get_string(tag, ID3_FRAME_COMMENT));
-#ifdef DEBUG
-            g_message("genre = %s", aud_tuple_get_string(tuple, FIELD_GENRE, NULL));
-#endif
+            AUDDBG("genre = %s\n", aud_tuple_get_string(tuple, FIELD_GENRE, NULL));
         }
         id3_file_close(id3file);
     } // id3file
     else { // no id3tag
-        realfn = g_filename_from_uri(filename, NULL, NULL);
-        __set_and_free(tuple, FIELD_FILE_NAME, NULL, g_path_get_basename(realfn ? realfn : filename));
-        __set_and_free(tuple, FIELD_FILE_PATH, NULL, g_path_get_dirname(realfn ? realfn : filename));
-        aud_tuple_associate_string(tuple, FIELD_FILE_EXT, NULL, extname(realfn ? realfn : filename));
-        g_free(realfn); realfn = NULL;
+        __set_and_free(tuple, FIELD_FILE_NAME, NULL, aud_uri_to_display_basename(filename));
+        __set_and_free(tuple, FIELD_FILE_PATH, NULL, aud_uri_to_display_dirname(filename));
+        aud_tuple_associate_string(tuple, FIELD_FILE_EXT, NULL, extname(filename));
         // length
-        {
-            char *dummy = NULL;
-            int length = 0;
-            if(aud_tuple_get_int(tuple, FIELD_LENGTH, NULL) == -1) {
-                audmad_get_song_length(filename, &length, fd);
-                aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, length);
-            }
-            g_free(dummy);
-        }
+        length = mad_timer_count(myinfo.duration, MAD_UNITS_MILLISECONDS);
+        aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, length);
     }
 
     aud_tuple_associate_string(tuple, FIELD_QUALITY, NULL, "lossy");
-    aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, "MPEG Audio (MP3)");
+    aud_tuple_associate_int(tuple, FIELD_BITRATE, NULL, myinfo.bitrate / 1000);
+
+    string = g_strdup_printf("MPEG-1 Audio Layer %d", myinfo.mpeg_layer);
+    aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, string);
+    g_free(string);
+
+    aud_tuple_associate_string(tuple, FIELD_MIMETYPE, NULL, "audio/mpeg");
+    
+    input_term(&myinfo);
 
     if(local_fd)
         aud_vfs_fclose(fd);
 
-#ifdef DEBUG
-    g_message("e: mad: audmad_get_song_tuple");
-#endif
+    AUDDBG("e: mad: audmad_get_song_tuple\n");
     return tuple;
 }
 
-static Tuple *audmad_get_song_tuple(char *filename)
+static Tuple *
+audmad_get_song_tuple(char *filename)
 {
     return __audmad_get_song_tuple(filename, NULL);
 }
 
-static Tuple *audmad_probe_for_tuple(char *filename, VFSFile *fd)
+static Tuple *
+audmad_probe_for_tuple(char *filename, VFSFile *fd)
 {
     if (!audmad_is_our_fd(filename, fd))
         return NULL;
@@ -833,12 +766,12 @@ InputPlugin mad_ip = {
     .seek = audmad_seek,
     .cleanup = audmad_cleanup,
     .get_song_info = audmad_get_song_info,
-    .file_info_box = audmad_get_file_info,
     .get_song_tuple = audmad_get_song_tuple,
     .is_our_file_from_vfs = audmad_is_our_fd,
     .vfs_extensions = fmts,
     .mseek = audmad_mseek,
-    .probe_for_tuple = audmad_probe_for_tuple
+    .probe_for_tuple = audmad_probe_for_tuple,
+    .update_song_tuple = audmad_update_song_tuple,
 };
 
 InputPlugin *madplug_iplist[] = { &mad_ip, NULL };
