@@ -8,13 +8,17 @@ using TagLib::Tag;
 using TagLib::String;
 using TagLib::APE::ItemListMap;
 
+const gchar *mpc_fmts[] = { "mpc", NULL };
+
 InputPlugin MpcPlugin = {
     NULL,           //File Handle               FILE* handle
     NULL,           //Filename                  char* filename
-    NULL,           //Name of Plugin            char* filename
+    (gchar *)"Musepack Audio Plugin",
     mpcOpenPlugin,  //Open Plugin               [CALLBACK]
+    NULL,           //Cleanup                   [UNUSED]
     mpcAboutBox,    //Show About box            [CALLBACK]
     mpcConfigBox,   //Show Configure box        [CALLBACK]
+    FALSE,          //Enabled/Disabled          [BOOLEAN]
     mpcIsOurFile,   //Check if it's our file    [CALLBACK]
     NULL,           //Scan the directory        [UNUSED]
     mpcPlay,        //Play                      [CALLBACK]
@@ -25,7 +29,6 @@ InputPlugin MpcPlugin = {
     mpcGetTime,     //Get Time                  [CALLBACK]
     NULL,           //Get Volume                [UNUSED]
     NULL,           //Set Volume                [UNUSED]
-    NULL,           //Close Plugin              [UNUSED]
     NULL,           //Obsolete                  [UNUSED]
     NULL,           //Visual plugins            add_vis_pcm(int time, AFormat fmt, int nch, int length, void *ptr)
     NULL,           //Set Info Settings         set_info(char *title, int length, int rate, int freq, int nch)
@@ -33,15 +36,16 @@ InputPlugin MpcPlugin = {
     mpcGetSongInfo, //Get Title String callback [CALLBACK]
     mpcFileInfoBox, //Show File Info Box        [CALLBACK]
     NULL,           //Output Plugin Handle      OutputPlugin output
-    mpcGetSongTuple,
+    mpcGetSongTuple,//Acquire tuple for song    [CALLBACK]
+    NULL,
+    NULL,
+    mpcIsOurFD,
+    (gchar **)mpc_fmts
 };
 
-extern "C"
-InputPlugin* get_iplugin_info()
-{
-    MpcPlugin.description = g_strdup_printf("Musepack Audio Plugin");
-    return &MpcPlugin;
-}
+InputPlugin *mpc_iplist[] = { &MpcPlugin, NULL };
+
+DECLARE_PLUGIN(musepack, NULL, NULL, mpc_iplist, NULL, NULL, NULL, NULL,NULL);
 
 static PluginConfig pluginConfig = {0};
 static Widgets      widgets      = {0};
@@ -56,34 +60,34 @@ GStaticMutex threadMutex = G_STATIC_MUTEX_INIT;
  * This _IS_ very sick, but it works. -nenolod
  */
 static mpc_int32_t
-vfs_fread_impl(void *data, void *ptr, mpc_int32_t size)
+aud_vfs_fread_impl(void *data, void *ptr, mpc_int32_t size)
 {
     mpc_reader_file *d = (mpc_reader_file *) data;
     VFSFile *file = (VFSFile *) d->file;
 
-    return (mpc_int32_t) vfs_fread(ptr, 1, size, file);
+    return (mpc_int32_t) aud_vfs_fread(ptr, 1, size, file);
 }
 
 static mpc_bool_t
-vfs_fseek_impl(void *data, mpc_int32_t offset)
+aud_vfs_fseek_impl(void *data, mpc_int32_t offset)
 {
     mpc_reader_file *d = (mpc_reader_file *) data;
     VFSFile *file = (VFSFile *) d->file;
 
-    return d->is_seekable ? vfs_fseek(file, offset, SEEK_SET) == 0 : FALSE;
+    return d->is_seekable ? aud_vfs_fseek(file, offset, SEEK_SET) == 0 : FALSE;
 }
 
 static mpc_int32_t
-vfs_ftell_impl(void *data)
+aud_vfs_ftell_impl(void *data)
 {
     mpc_reader_file *d = (mpc_reader_file *) data;
     VFSFile *file = (VFSFile *) d->file;
 
-    return vfs_ftell(file);
+    return aud_vfs_ftell(file);
 }
 
 static mpc_int32_t
-vfs_getsize_impl(void *data)
+aud_vfs_getsize_impl(void *data)
 {
     mpc_reader_file *d = (mpc_reader_file *) data;
 
@@ -91,7 +95,7 @@ vfs_getsize_impl(void *data)
 }
 
 static mpc_bool_t
-vfs_canseek_impl(void *data)
+aud_vfs_canseek_impl(void *data)
 {
     mpc_reader_file *d = (mpc_reader_file *) data;
 
@@ -106,30 +110,30 @@ vfs_canseek_impl(void *data)
 void
 mpc_reader_setup_file_vfs(mpc_reader_file *p_reader, VFSFile *input)
 {
-    p_reader->reader.seek = vfs_fseek_impl;
-    p_reader->reader.read = vfs_fread_impl;
-    p_reader->reader.tell = vfs_ftell_impl;
-    p_reader->reader.get_size = vfs_getsize_impl;
-    p_reader->reader.canseek = vfs_canseek_impl;
+    p_reader->reader.seek = aud_vfs_fseek_impl;
+    p_reader->reader.read = aud_vfs_fread_impl;
+    p_reader->reader.tell = aud_vfs_ftell_impl;
+    p_reader->reader.get_size = aud_vfs_getsize_impl;
+    p_reader->reader.canseek = aud_vfs_canseek_impl;
     p_reader->reader.data = p_reader;
 
     p_reader->file = (FILE *) input; // no worries, it gets cast back -nenolod
     p_reader->is_seekable = TRUE;    // XXX streams
 
-    vfs_fseek(input, 0, SEEK_END);
-    p_reader->file_size = vfs_ftell(input);
-    vfs_fseek(input, 0, SEEK_SET);
+    aud_vfs_fseek(input, 0, SEEK_END);
+    p_reader->file_size = aud_vfs_ftell(input);
+    aud_vfs_fseek(input, 0, SEEK_SET);
 }
 
 static void mpcOpenPlugin()
 {
     ConfigDb *cfg;
-    cfg = bmp_cfg_db_open();
-    bmp_cfg_db_get_bool(cfg, "musepack", "clipPrevention", &pluginConfig.clipPrevention);
-    bmp_cfg_db_get_bool(cfg, "musepack", "albumGain",      &pluginConfig.albumGain);
-    bmp_cfg_db_get_bool(cfg, "musepack", "dynamicBitrate", &pluginConfig.dynamicBitrate);
-    bmp_cfg_db_get_bool(cfg, "musepack", "replaygain",     &pluginConfig.replaygain);
-    bmp_cfg_db_close(cfg);
+    cfg = aud_cfg_db_open();
+    aud_cfg_db_get_bool(cfg, "musepack", "clipPrevention", &pluginConfig.clipPrevention);
+    aud_cfg_db_get_bool(cfg, "musepack", "albumGain",      &pluginConfig.albumGain);
+    aud_cfg_db_get_bool(cfg, "musepack", "dynamicBitrate", &pluginConfig.dynamicBitrate);
+    aud_cfg_db_get_bool(cfg, "musepack", "replaygain",     &pluginConfig.replaygain);
+    aud_cfg_db_close(cfg);
 }
 
 static void mpcAboutBox()
@@ -139,10 +143,10 @@ static void mpcAboutBox()
         gdk_window_raise(aboutBox->window);
     else
     {
-        char* titleText      = g_strdup_printf("Musepack Decoder Plugin 1.2");
-        char* contentText = "Plugin code by\nBenoit Amiaux\nMartin Spuler\nKuniklo\n\nGet latest version at http://musepack.net\n";
-        char* buttonText  = "Nevermind";
-        aboutBox = xmms_show_message(titleText, contentText, buttonText, FALSE, NULL, NULL);
+        char* titleText      = g_strdup_printf(_("Musepack Decoder Plugin 1.2"));
+        const char* contentText = _("Plugin code by\nBenoit Amiaux\nMartin Spuler\nKuniklo\n\nGet latest version at http://musepack.net\n");
+        const char* buttonText  = _("Nevermind");
+        aboutBox = audacious_info_dialog(titleText, contentText, buttonText, FALSE, NULL, NULL);
         widgets.aboutBox = aboutBox;
         g_signal_connect(G_OBJECT(aboutBox), "destroy", G_CALLBACK(gtk_widget_destroyed), &widgets.aboutBox);
     }
@@ -159,7 +163,7 @@ static void mpcConfigBox()
         gtk_window_set_type_hint(GTK_WINDOW(configBox), GDK_WINDOW_TYPE_HINT_DIALOG);
         widgets.configBox = configBox;
         g_signal_connect(G_OBJECT(configBox), "destroy", G_CALLBACK(gtk_widget_destroyed), &widgets.configBox);
-        gtk_window_set_title(GTK_WINDOW(configBox), "Musepack Decoder Configuration");
+        gtk_window_set_title(GTK_WINDOW(configBox), _("Musepack Decoder Configuration"));
         gtk_window_set_policy(GTK_WINDOW(configBox), FALSE, FALSE, FALSE);
         gtk_container_border_width(GTK_CONTAINER(configBox), 10);
 
@@ -169,38 +173,38 @@ static void mpcConfigBox()
         gtk_container_add(GTK_CONTAINER(configBox), vbox);
 
         //General Settings Tab
-        GtkWidget* generalSet = gtk_frame_new("General Settings");
+        GtkWidget* generalSet = gtk_frame_new(_("General Settings"));
         gtk_container_border_width(GTK_CONTAINER(generalSet), 5);
 
         GtkWidget* gSvbox = gtk_vbox_new(FALSE, 10);
         gtk_container_border_width(GTK_CONTAINER(gSvbox), 5);
         gtk_container_add(GTK_CONTAINER(generalSet), gSvbox);
 
-        GtkWidget* bitrateCheck = gtk_check_button_new_with_label("Enable Dynamic Bitrate Display");
+        GtkWidget* bitrateCheck = gtk_check_button_new_with_label(_("Enable Dynamic Bitrate Display"));
         widgets.bitrateCheck = bitrateCheck;
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bitrateCheck), pluginConfig.dynamicBitrate);
         gtk_box_pack_start(GTK_BOX(gSvbox), bitrateCheck, FALSE, FALSE, 0);
-        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), generalSet, gtk_label_new("Plugin"));
+        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), generalSet, gtk_label_new(_("Plugin")));
 
         //ReplayGain Settings Tab
-        GtkWidget* replaygainSet = gtk_frame_new("ReplayGain Settings");
+        GtkWidget* replaygainSet = gtk_frame_new(_("ReplayGain Settings"));
         gtk_container_border_width(GTK_CONTAINER(replaygainSet), 5);
 
         GtkWidget* rSVbox = gtk_vbox_new(FALSE, 10);
         gtk_container_border_width(GTK_CONTAINER(rSVbox), 5);
         gtk_container_add(GTK_CONTAINER(replaygainSet), rSVbox);
 
-        GtkWidget* clippingCheck = gtk_check_button_new_with_label("Enable Clipping Prevention");
+        GtkWidget* clippingCheck = gtk_check_button_new_with_label(_("Enable Clipping Prevention"));
         widgets.clippingCheck = clippingCheck;
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(clippingCheck), pluginConfig.clipPrevention);
         gtk_box_pack_start(GTK_BOX(rSVbox), clippingCheck, FALSE, FALSE, 0);
 
-        GtkWidget* replaygainCheck = gtk_check_button_new_with_label("Enable ReplayGain");
+        GtkWidget* replaygainCheck = gtk_check_button_new_with_label(_("Enable ReplayGain"));
         widgets.replaygainCheck = replaygainCheck;
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(replaygainCheck), pluginConfig.replaygain);
         gtk_box_pack_start(GTK_BOX(rSVbox), replaygainCheck, FALSE, FALSE, 0);
 
-        GtkWidget* replaygainType = gtk_frame_new("ReplayGain Type");
+        GtkWidget* replaygainType = gtk_frame_new(_("ReplayGain Type"));
         gtk_box_pack_start(GTK_BOX(rSVbox), replaygainType, FALSE, FALSE, 0);
         g_signal_connect(G_OBJECT(replaygainCheck), "toggled", G_CALLBACK(toggleSwitch), replaygainType);
 
@@ -208,16 +212,16 @@ static void mpcConfigBox()
         gtk_container_set_border_width(GTK_CONTAINER(rgVbox), 5);
         gtk_container_add(GTK_CONTAINER(replaygainType), rgVbox);
 
-        GtkWidget* trackCheck = gtk_radio_button_new_with_label(NULL, "Use Track Gain");
+        GtkWidget* trackCheck = gtk_radio_button_new_with_label(NULL, _("Use Track Gain"));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(trackCheck), !pluginConfig.albumGain);
         gtk_box_pack_start(GTK_BOX(rgVbox), trackCheck, FALSE, FALSE, 0);
 
-        GtkWidget* albumCheck = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(trackCheck)), "Use Album Gain");
+        GtkWidget* albumCheck = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(trackCheck)), _("Use Album Gain"));
         widgets.albumCheck = albumCheck;
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(albumCheck), pluginConfig.albumGain);
         gtk_box_pack_start(GTK_BOX(rgVbox), albumCheck, FALSE, FALSE, 0);
         gtk_widget_set_sensitive(replaygainType, pluginConfig.replaygain);
-        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), replaygainSet, gtk_label_new("ReplayGain"));
+        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), replaygainSet, gtk_label_new(_("ReplayGain")));
 
         //Buttons
         GtkWidget* buttonBox = gtk_hbutton_box_new();
@@ -225,12 +229,12 @@ static void mpcConfigBox()
         gtk_button_box_set_spacing(GTK_BUTTON_BOX(buttonBox), 5);
         gtk_box_pack_start(GTK_BOX(vbox), buttonBox, FALSE, FALSE, 0);
 
-        GtkWidget* okButton = gtk_button_new_with_label("Ok");
+        GtkWidget* okButton = gtk_button_new_with_label(_("Ok"));
         g_signal_connect(G_OBJECT(okButton), "clicked", G_CALLBACK(saveConfigBox), NULL);
         GTK_WIDGET_SET_FLAGS(okButton, GTK_CAN_DEFAULT);
         gtk_box_pack_start(GTK_BOX(buttonBox), okButton, TRUE, TRUE, 0);
 
-        GtkWidget* cancelButton = gtk_button_new_with_label("Cancel");
+        GtkWidget* cancelButton = gtk_button_new_with_label(_("Cancel"));
         g_signal_connect_swapped(G_OBJECT(cancelButton), "clicked", G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(widgets.configBox));
         GTK_WIDGET_SET_FLAGS(cancelButton, GTK_CAN_DEFAULT);
         gtk_widget_grab_default(cancelButton);
@@ -259,43 +263,54 @@ static void saveConfigBox(GtkWidget* p_Widget, gpointer p_Data)
     tb = GTK_TOGGLE_BUTTON(widgets.albumCheck);
     pluginConfig.albumGain = gtk_toggle_button_get_active(tb);
 
-    cfg = bmp_cfg_db_open();
+    cfg = aud_cfg_db_open();
 
-    bmp_cfg_db_set_bool(cfg, "musepack", "clipPrevention", pluginConfig.clipPrevention);
-    bmp_cfg_db_set_bool(cfg, "musepack", "albumGain",      pluginConfig.albumGain);
-    bmp_cfg_db_set_bool(cfg, "musepack", "dynamicBitrate", pluginConfig.dynamicBitrate);
-    bmp_cfg_db_set_bool(cfg, "musepack", "replaygain",     pluginConfig.replaygain);
+    aud_cfg_db_set_bool(cfg, "musepack", "clipPrevention", pluginConfig.clipPrevention);
+    aud_cfg_db_set_bool(cfg, "musepack", "albumGain",      pluginConfig.albumGain);
+    aud_cfg_db_set_bool(cfg, "musepack", "dynamicBitrate", pluginConfig.dynamicBitrate);
+    aud_cfg_db_set_bool(cfg, "musepack", "replaygain",     pluginConfig.replaygain);
 
-    bmp_cfg_db_close(cfg);
+    aud_cfg_db_close(cfg);
 
     gtk_widget_destroy (widgets.configBox);
 }
 
-static int mpcIsOurFile(char* p_Filename)
+static gint mpcIsOurFile(gchar* p_Filename)
 {
-   VFSFile *file;
-   gchar magic[3];
-   if ((file = vfs_fopen(p_Filename, "rb"))) {
-       vfs_fread(magic, 1, 3, file);
-       if (!strncmp(magic, "MP+", 3)) {
-            vfs_fclose(file);
-            return 1;
-       }
-       vfs_fclose(file);
-   }
-   return 0;
+    VFSFile *file;
+    gchar magic[3];
+    if ((file = aud_vfs_fopen(p_Filename, "rb"))) {
+        aud_vfs_fread(magic, 1, 3, file);
+        if (!memcmp(magic, "MP+", 3)) {
+             aud_vfs_fclose(file);
+             return 1;
+        }
+        aud_vfs_fclose(file);
+    }
+    return 0;
 }
 
-static void mpcPlay(char* p_Filename)
+static gint mpcIsOurFD(gchar* p_Filename, VFSFile* file)
+{
+    gchar magic[3];
+    aud_vfs_fread(magic, 1, 3, file);
+    if (!memcmp(magic, "MP+", 3))
+        return 1;
+    return 0;
+}
+
+static void mpcPlay(InputPlayback *data)
 {
     mpcDecoder.offset   = -1;
     mpcDecoder.isAlive  = true;
     mpcDecoder.isOutput = false;
     mpcDecoder.isPause  = false;
-    threadHandle = g_thread_create(GThreadFunc(decodeStream), (void *) g_strdup(p_Filename), TRUE, NULL);
+    threadHandle = g_thread_self();
+    data->set_pb_ready(data);
+    decodeStream(data);
 }
 
-static void mpcStop()
+static void mpcStop(InputPlayback *data)
 {
     setAlive(false);
     if (threadHandle)
@@ -303,60 +318,55 @@ static void mpcStop()
         g_thread_join(threadHandle);
         if (mpcDecoder.isOutput)
         {
-            MpcPlugin.output->buffer_free();
-            MpcPlugin.output->close_audio();
+            data->output->buffer_free();
+            data->output->close_audio();
             mpcDecoder.isOutput = false;
         }
     }
 }
 
-static void mpcPause(short p_Pause)
+static void mpcPause(InputPlayback *data, short p_Pause)
 {
     lockAcquire();
     mpcDecoder.isPause = p_Pause;
-    MpcPlugin.output->pause(p_Pause);
+    data->output->pause(p_Pause);
     lockRelease();
 }
 
-static void mpcSeek(int p_Offset)
+static void mpcSeek(InputPlayback *data, int p_Offset)
 {
     lockAcquire();
     mpcDecoder.offset = static_cast<double> (p_Offset);
-    MpcPlugin.output->flush(1000 * p_Offset);
+    data->output->flush(1000 * p_Offset);
     lockRelease();
 }
 
-static int mpcGetTime()
+static gint mpcGetTime(InputPlayback *data)
 {
     if(!isAlive())
         return -1;
-    return MpcPlugin.output->output_time();
+    return data->output->output_time();
 }
 
-static TitleInput *mpcGetSongTuple(char* p_Filename)
+static Tuple *mpcGetSongTuple(gchar* p_Filename)
 {
-    VFSFile *input = vfs_fopen(p_Filename, "rb");
-    TitleInput *tuple = NULL;
+    VFSFile *input = aud_vfs_fopen(p_Filename, "rb");
+    Tuple *tuple = NULL;
 
     if(input)
     {
-        tuple = bmp_title_input_new();
-        gchar *filename_proxy = g_strdup(p_Filename);
-
-        tuple->file_name = g_path_get_basename(filename_proxy);
-        tuple->file_path = g_path_get_dirname(filename_proxy);
-        tuple->file_ext = "mpc";		// XXX: I can't be assed. -nenolod
+        tuple = aud_tuple_new_from_filename(p_Filename);
 
         MpcInfo tags = getTags(p_Filename);
 
-        tuple->date         = g_strdup(tags.date);
-        tuple->track_name   = g_strdup(tags.title);
-        tuple->performer    = g_strdup(tags.artist);
-        tuple->album_name   = g_strdup(tags.album);
-        tuple->track_number = tags.track;
-        tuple->year         = tags.year;
-        tuple->genre        = g_strdup(tags.genre);
-        tuple->comment      = g_strdup(tags.comment);
+        aud_tuple_associate_string(tuple, FIELD_DATE, NULL, tags.date);
+        aud_tuple_associate_string(tuple, FIELD_TITLE, NULL, tags.title);
+        aud_tuple_associate_string(tuple, FIELD_ARTIST, NULL, tags.artist);
+        aud_tuple_associate_string(tuple, FIELD_ALBUM, NULL, tags.album);
+        aud_tuple_associate_int(tuple, FIELD_TRACK_NUMBER, NULL, tags.track);
+        aud_tuple_associate_int(tuple, FIELD_YEAR, NULL, tags.year);
+        aud_tuple_associate_string(tuple, FIELD_GENRE, NULL, tags.genre);
+        aud_tuple_associate_string(tuple, FIELD_COMMENT, NULL, tags.comment);
 
         freeTags(tags);
 
@@ -365,14 +375,23 @@ static TitleInput *mpcGetSongTuple(char* p_Filename)
         mpc_reader_setup_file_vfs(&reader, input);
         mpc_streaminfo_read(&info, &reader.reader);
 
-        tuple->length = static_cast<int> (1000 * mpc_streaminfo_get_length(&info));
-        vfs_fclose(input);
+        aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, static_cast<int> (1000 * mpc_streaminfo_get_length(&info)));
+
+        gchar *scratch = g_strdup_printf("Musepack v%d (encoder %s)", info.stream_version, info.encoder);
+        aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, scratch);
+        g_free(scratch);
+
+        scratch = g_strdup_printf("lossy (%s)", info.profile_name);
+        aud_tuple_associate_string(tuple, FIELD_QUALITY, NULL, scratch);
+        g_free(scratch);
+
+        aud_vfs_fclose(input);
     }
     else
     {
-        char* temp = g_strdup_printf("[xmms-musepack] mpcGetSongInfo is unable to open %s\n", p_Filename);
+        gchar* temp = g_strdup_printf("[xmms-musepack] mpcGetSongInfo is unable to open %s\n", p_Filename);
         perror(temp);
-        free(temp);
+        g_free(temp);
     }
 
     return tuple;
@@ -380,7 +399,7 @@ static TitleInput *mpcGetSongTuple(char* p_Filename)
 
 static void mpcGetSongInfo(char* p_Filename, char** p_Title, int* p_Length)
 {
-    VFSFile *input = vfs_fopen(p_Filename, "rb");
+    VFSFile *input = aud_vfs_fopen(p_Filename, "rb");
     if(input)
     {
         MpcInfo tags = getTags(p_Filename);
@@ -391,29 +410,31 @@ static void mpcGetSongInfo(char* p_Filename, char** p_Title, int* p_Length)
         mpc_reader_setup_file_vfs(&reader, input);
         mpc_streaminfo_read(&info, &reader.reader);
         *p_Length = static_cast<int> (1000 * mpc_streaminfo_get_length(&info));
-        vfs_fclose(input);
+        aud_vfs_fclose(input);
     }
     else
     {
-        char* temp = g_strdup_printf("[xmms-musepack] mpcGetSongInfo is unable to open %s\n", p_Filename);
+        gchar* temp = g_strdup_printf("[xmms-musepack] mpcGetSongInfo is unable to open %s\n", p_Filename);
         perror(temp);
-        free(temp);
+        g_free(temp);
     }
 }
 
 static void freeTags(MpcInfo& tags)
 {
-    free(tags.title);
-    free(tags.artist);
-    free(tags.album);
-    free(tags.comment);
-    free(tags.genre);
-    free(tags.date);
+    g_free(tags.title);
+    g_free(tags.artist);
+    g_free(tags.album);
+    g_free(tags.comment);
+    g_free(tags.genre);
+    g_free(tags.date);
 }
 
-static MpcInfo getTags(const char* p_Filename)
+static MpcInfo getTags(const gchar* p_Filename)
 {
-    File oFile(p_Filename, false);
+    gchar *pRealFilename = g_filename_from_uri(p_Filename, NULL, NULL);
+    File oFile(pRealFilename ? pRealFilename : p_Filename, false);
+    g_free(pRealFilename);
     Tag* poTag = oFile.tag();
     MpcInfo tags = {0};
     tags.title   = g_strdup(poTag->title().toCString(true));
@@ -428,6 +449,7 @@ static MpcInfo getTags(const char* p_Filename)
     REMOVE_NONEXISTANT_TAG(tags.comment);
     tags.year    = poTag->year();
     tags.track   = poTag->track();
+#if 0
     TagLib::APE::Tag* ape = oFile.APETag(false);
     if(ape)
     {
@@ -441,6 +463,7 @@ static MpcInfo getTags(const char* p_Filename)
             tags.date = g_strdup_printf("%d", tags.year);
         }
     }
+#endif
     return tags;
 }
 
@@ -465,7 +488,7 @@ static void mpcFileInfoBox(char* p_Filename)
         GtkWidget* filenameHbox = gtk_hbox_new(FALSE, 5);
         gtk_box_pack_start(GTK_BOX(iVbox), filenameHbox, FALSE, TRUE, 0);
 
-        GtkWidget* fileLabel = gtk_label_new("Filename:");
+        GtkWidget* fileLabel = gtk_label_new(_("Filename:"));
         gtk_box_pack_start(GTK_BOX(filenameHbox), fileLabel, FALSE, TRUE, 0);
 
         GtkWidget* fileEntry = gtk_entry_new();
@@ -480,7 +503,7 @@ static void mpcFileInfoBox(char* p_Filename)
         gtk_box_pack_start(GTK_BOX(iHbox), leftBox, FALSE, FALSE, 0);
 
         //Tag labels
-        GtkWidget* tagFrame = gtk_frame_new("Musepack Tag");
+        GtkWidget* tagFrame = gtk_frame_new(_("Musepack Tag"));
         gtk_box_pack_start(GTK_BOX(leftBox), tagFrame, FALSE, FALSE, 0);
         gtk_widget_set_sensitive(tagFrame, TRUE);
 
@@ -488,33 +511,33 @@ static void mpcFileInfoBox(char* p_Filename)
         gtk_container_set_border_width(GTK_CONTAINER(iTable), 5);
         gtk_container_add(GTK_CONTAINER(tagFrame), iTable);
 
-        mpcGtkTagLabel("Title:", 0, 1, 0, 1, iTable);
+        mpcGtkTagLabel(_("Title:"), 0, 1, 0, 1, iTable);
         GtkWidget* titleEntry = mpcGtkTagEntry(1, 4, 0, 1, 0, iTable);
         widgets.titleEntry = titleEntry;
 
-        mpcGtkTagLabel("Artist:", 0, 1, 1, 2, iTable);
+        mpcGtkTagLabel(_("Artist:"), 0, 1, 1, 2, iTable);
         GtkWidget* artistEntry = mpcGtkTagEntry(1, 4, 1, 2, 0, iTable);
         widgets.artistEntry = artistEntry;
 
-        mpcGtkTagLabel("Album:", 0, 1, 2, 3, iTable);
+        mpcGtkTagLabel(_("Album:"), 0, 1, 2, 3, iTable);
         GtkWidget* albumEntry = mpcGtkTagEntry(1, 4, 2, 3, 0, iTable);
         widgets.albumEntry = albumEntry;
 
-        mpcGtkTagLabel("Comment:", 0, 1, 3, 4, iTable);
+        mpcGtkTagLabel(_("Comment:"), 0, 1, 3, 4, iTable);
         GtkWidget* commentEntry = mpcGtkTagEntry(1, 4, 3, 4, 0, iTable);
         widgets.commentEntry = commentEntry;
 
-        mpcGtkTagLabel("Year:", 0, 1, 4, 5, iTable);
+        mpcGtkTagLabel(_("Year:"), 0, 1, 4, 5, iTable);
         GtkWidget* yearEntry = mpcGtkTagEntry(1, 2, 4, 5, 4, iTable);
         widgets.yearEntry = yearEntry;
         gtk_widget_set_usize(yearEntry, 4, -1);
 
-        mpcGtkTagLabel("Track:", 2, 3, 4, 5, iTable);
+        mpcGtkTagLabel(_("Track:"), 2, 3, 4, 5, iTable);
         GtkWidget* trackEntry = mpcGtkTagEntry(3, 4, 4, 5, 4, iTable);
         widgets.trackEntry = trackEntry;
         gtk_widget_set_usize(trackEntry, 3, -1);
 
-        mpcGtkTagLabel("Genre:", 0, 1, 5, 6, iTable);
+        mpcGtkTagLabel(_("Genre:"), 0, 1, 5, 6, iTable);
         GtkWidget* genreEntry = mpcGtkTagEntry(1, 4, 5, 6, 0, iTable);
         widgets.genreEntry = genreEntry;
         gtk_widget_set_usize(genreEntry, 20, -1);
@@ -525,18 +548,18 @@ static void mpcFileInfoBox(char* p_Filename)
         gtk_button_box_set_spacing(GTK_BUTTON_BOX(buttonBox), 5);
         gtk_box_pack_start(GTK_BOX(leftBox), buttonBox, FALSE, FALSE, 0);
 
-        GtkWidget* saveButton = mpcGtkButton("Save", buttonBox);
+        GtkWidget* saveButton = mpcGtkButton(_("Save"), buttonBox);
         g_signal_connect(G_OBJECT(saveButton), "clicked", G_CALLBACK(saveTags), NULL);
 
-        GtkWidget* removeButton = mpcGtkButton("Remove Tag", buttonBox);
+        GtkWidget* removeButton = mpcGtkButton(_("Remove Tag"), buttonBox);
         g_signal_connect_swapped(G_OBJECT(removeButton), "clicked", G_CALLBACK(removeTags), NULL);
 
-        GtkWidget* cancelButton = mpcGtkButton("Cancel", buttonBox);
+        GtkWidget* cancelButton = mpcGtkButton(_("Cancel"), buttonBox);
         g_signal_connect_swapped(G_OBJECT(cancelButton), "clicked", G_CALLBACK(closeInfoBox), NULL);
         gtk_widget_grab_default(cancelButton);
 
         //File information
-        GtkWidget* infoFrame = gtk_frame_new("Musepack Info");
+        GtkWidget* infoFrame = gtk_frame_new(_("Musepack Info"));
         gtk_box_pack_start(GTK_BOX(iHbox), infoFrame, FALSE, FALSE, 0);
 
         GtkWidget* infoVbox = gtk_vbox_new(FALSE, 5);
@@ -557,7 +580,7 @@ static void mpcFileInfoBox(char* p_Filename)
         GtkWidget* albumPeakLabel = mpcGtkLabel(infoVbox);
         GtkWidget* albumGainLabel = mpcGtkLabel(infoVbox);
 
-        VFSFile *input = vfs_fopen(p_Filename, "rb");
+        VFSFile *input = aud_vfs_fopen(p_Filename, "rb");
         if(input)
         {
             mpc_streaminfo info;
@@ -565,22 +588,22 @@ static void mpcFileInfoBox(char* p_Filename)
             mpc_reader_setup_file_vfs(&reader, input);
             mpc_streaminfo_read(&info, &reader.reader);
 
-            int time = static_cast<int> (mpc_streaminfo_get_length(&info));
-            int minutes = time / 60;
-            int seconds = time % 60;
+            gint time = static_cast<int> (mpc_streaminfo_get_length(&info));
+            gint minutes = time / 60;
+            gint seconds = time % 60;
 
-            mpcGtkPrintLabel(streamLabel,    "Streamversion %d", info.stream_version);
-            mpcGtkPrintLabel(encoderLabel,   "Encoder: \%s", info.encoder);
-            mpcGtkPrintLabel(profileLabel,   "Profile: \%s", info.profile_name);
-            mpcGtkPrintLabel(bitrateLabel,   "Average bitrate: \%6.1f kbps", info.average_bitrate * 1.e-3);
-            mpcGtkPrintLabel(rateLabel,      "Samplerate: \%d Hz", info.sample_freq);
-            mpcGtkPrintLabel(channelsLabel,  "Channels: \%d", info.channels);
-            mpcGtkPrintLabel(lengthLabel,    "Length: \%d:\%.2d", minutes, seconds);
-            mpcGtkPrintLabel(fileSizeLabel,  "File size: \%d Bytes", info.total_file_length);
-            mpcGtkPrintLabel(trackPeakLabel, "Track Peak: \%5u", info.peak_title);
-            mpcGtkPrintLabel(trackGainLabel, "Track Gain: \%-+2.2f dB", 0.01 * info.gain_title);
-            mpcGtkPrintLabel(albumPeakLabel, "Album Peak: \%5u", info.peak_album);
-            mpcGtkPrintLabel(albumGainLabel, "Album Gain: \%-+5.2f dB", 0.01 * info.gain_album);
+            mpcGtkPrintLabel(streamLabel,    _("Streamversion %d"), info.stream_version);
+            mpcGtkPrintLabel(encoderLabel,   _("Encoder: %s"), info.encoder);
+            mpcGtkPrintLabel(profileLabel,   _("Profile: %s"), info.profile_name);
+            mpcGtkPrintLabel(bitrateLabel,   _("Average bitrate: %6.1f kbps"), info.average_bitrate * 1.e-3);
+            mpcGtkPrintLabel(rateLabel,      _("Samplerate: %d Hz"), info.sample_freq);
+            mpcGtkPrintLabel(channelsLabel,  _("Channels: %d"), info.channels);
+            mpcGtkPrintLabel(lengthLabel,    _("Length: %d:\%.2d"), minutes, seconds);
+            mpcGtkPrintLabel(fileSizeLabel,  _("File size: %d Bytes"), info.total_file_length);
+            mpcGtkPrintLabel(trackPeakLabel, _("Track Peak: %5u"), info.peak_title);
+            mpcGtkPrintLabel(trackGainLabel, _("Track Gain: %-+2.2f dB"), 0.01 * info.gain_title);
+            mpcGtkPrintLabel(albumPeakLabel, _("Album Peak: %5u"), info.peak_album);
+            mpcGtkPrintLabel(albumGainLabel, _("Album Gain: %-+5.2f dB"), 0.01 * info.gain_album);
 
             MpcInfo tags = getTags(p_Filename);
             gtk_entry_set_text(GTK_ENTRY(titleEntry),   tags.title);
@@ -588,48 +611,48 @@ static void mpcFileInfoBox(char* p_Filename)
             gtk_entry_set_text(GTK_ENTRY(albumEntry),   tags.album);
             gtk_entry_set_text(GTK_ENTRY(commentEntry), tags.comment);
             gtk_entry_set_text(GTK_ENTRY(genreEntry),   tags.genre);
-            char* entry = g_strdup_printf ("%d", tags.track);
+            gchar* entry = g_strdup_printf ("%d", tags.track);
             gtk_entry_set_text(GTK_ENTRY(trackEntry), entry);
-            free(entry);
+            g_free(entry);
             entry = g_strdup_printf ("%d", tags.year);
             gtk_entry_set_text(GTK_ENTRY(yearEntry), entry);
-            free(entry);
+            g_free(entry);
             entry = g_filename_display_name(p_Filename);
             gtk_entry_set_text(GTK_ENTRY(fileEntry), entry);
-            free(entry);
+            g_free(entry);
             freeTags(tags);
-            vfs_fclose(input);
+            aud_vfs_fclose(input);
         }
         else
         {
-            char* temp = g_strdup_printf("[xmms-musepack] mpcFileInfoBox is unable to read tags from %s", p_Filename);
+            gchar* temp = g_strdup_printf("[xmms-musepack] mpcFileInfoBox is unable to read tags from %s", p_Filename);
             perror(temp);
-            free(temp);
+            g_free(temp);
         }
 
-	char* name = g_filename_display_basename(p_Filename);
-        char* text = g_strdup_printf("File Info - %s", name);
-        free(name);
+	gchar* name = g_filename_display_basename(p_Filename);
+        gchar* text = g_strdup_printf(_("File Info - %s"), name);
+        g_free(name);
         gtk_window_set_title(GTK_WINDOW(infoBox), text);
-        free(text);
+        g_free(text);
 
         gtk_widget_show_all(infoBox);
     }
 }
 
-static void mpcGtkPrintLabel(GtkWidget* widget, char* format,...)
+static void mpcGtkPrintLabel(GtkWidget* widget, const char* format,...)
 {
     va_list args;
 
     va_start(args, format);
-    char* temp = g_strdup_vprintf(format, args);
+    gchar* temp = g_strdup_vprintf(format, args);
     va_end(args);
 
     gtk_label_set_text(GTK_LABEL(widget), temp);
-    free(temp);
+    g_free(temp);
 }
 
-static GtkWidget* mpcGtkTagLabel(char* p_Text, int a, int b, int c, int d, GtkWidget* p_Box)
+static GtkWidget* mpcGtkTagLabel(const char* p_Text, int a, int b, int c, int d, GtkWidget* p_Box)
 {
     GtkWidget* label = gtk_label_new(p_Text);
     gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
@@ -659,7 +682,7 @@ static GtkWidget* mpcGtkLabel(GtkWidget* p_Box)
     return label;
 }
 
-static GtkWidget* mpcGtkButton(char* p_Text, GtkWidget* p_Box)
+static GtkWidget* mpcGtkButton(const char* p_Text, GtkWidget* p_Box)
 {
     GtkWidget* button = gtk_button_new_with_label(p_Text);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
@@ -680,11 +703,11 @@ static void saveTags(GtkWidget* w, gpointer data)
     File oFile(gtk_entry_get_text(GTK_ENTRY(widgets.fileEntry)));
     Tag* poTag = oFile.tag();
 
-    char* cAlbum   = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.albumEntry)));
-    char* cArtist  = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.artistEntry)));
-    char* cTitle   = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.titleEntry)));
-    char* cGenre   = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.genreEntry)));
-    char* cComment = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.commentEntry)));
+    gchar* cAlbum   = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.albumEntry)));
+    gchar* cArtist  = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.artistEntry)));
+    gchar* cTitle   = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.titleEntry)));
+    gchar* cGenre   = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.genreEntry)));
+    gchar* cComment = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.commentEntry)));
 
     const String album   = String(cAlbum,   TagLib::String::UTF8);
     const String artist  = String(cArtist,  TagLib::String::UTF8);
@@ -700,11 +723,11 @@ static void saveTags(GtkWidget* w, gpointer data)
     poTag->setYear(atoi(gtk_entry_get_text(GTK_ENTRY(widgets.yearEntry))));
     poTag->setTrack(atoi(gtk_entry_get_text(GTK_ENTRY(widgets.trackEntry))));
 
-    free(cAlbum);
-    free(cArtist);
-    free(cTitle);
-    free(cGenre);
-    free(cComment);
+    g_free(cAlbum);
+    g_free(cArtist);
+    g_free(cTitle);
+    g_free(cGenre);
+    g_free(cComment);
 
     oFile.save();
     closeInfoBox(NULL, NULL);
@@ -716,48 +739,42 @@ static void closeInfoBox(GtkWidget* w, gpointer data)
     widgets.infoBox = NULL;
 }
 
-static char* mpcGenerateTitle(const MpcInfo& p_Tags, char* p_Filename)
+static char* mpcGenerateTitle(const MpcInfo& p_Tags, gchar* p_Filename)
 {
-    TitleInput* tuple = mpcGetSongTuple(p_Filename);
+    Tuple* tuple = mpcGetSongTuple(p_Filename);
 
-    char* title = xmms_get_titlestring (xmms_get_gentitle_format(), tuple);
-    if(!title)
-        title = g_strdup(tuple->file_name);
-    else if (!*title)
-        title = g_strdup(tuple->file_name);
+    gchar* title = aud_tuple_formatter_make_title_string(tuple, aud_get_gentitle_format());
 
-    bmp_title_input_free(tuple);
+    aud_tuple_free((void *) tuple);
     return title;
 }
 
-static void* endThread(char* p_FileName, VFSFile * p_FileHandle, bool release)
+static void* endThread(gchar* p_FileName, VFSFile * p_FileHandle, bool release)
 {
-    free(p_FileName);
     if(release)
         lockRelease();
     if(mpcDecoder.isError)
     {
         perror(mpcDecoder.isError);
-        free(mpcDecoder.isError);
+        g_free(mpcDecoder.isError);
         mpcDecoder.isError = NULL;
     }
     setAlive(false);
     if(p_FileHandle)
-        vfs_fclose(p_FileHandle);
+        aud_vfs_fclose(p_FileHandle);
     if(track.display)
     {
-        free(track.display);
+        g_free(track.display);
         track.display = NULL;
     }
-    g_thread_exit(NULL);
     return 0;
 }
 
-static void* decodeStream(void* data)
+static void* decodeStream(InputPlayback *data)
 {
     lockAcquire();
-    char* filename = static_cast<char*> (data);
-    VFSFile *input = vfs_fopen(filename, "rb");
+    gchar* filename = data->filename;
+    VFSFile *input = aud_vfs_fopen(filename, "rb");
     if (!input)
     {
         mpcDecoder.isError = g_strdup_printf("[xmms-musepack] decodeStream is unable to open %s", filename);
@@ -782,7 +799,7 @@ static void* decodeStream(void* data)
     track.channels   = info.channels;
     freeTags(tags);
 
-    MpcPlugin.set_info(track.display, track.length, track.bitrate, track.sampleFreq, track.channels);
+    data->set_params(data, track.display, track.length, track.bitrate, track.sampleFreq, track.channels);
 
     mpc_decoder decoder;
     mpc_decoder_setup(&decoder, &reader.reader);
@@ -809,7 +826,7 @@ static void* decodeStream(void* data)
 
     lockRelease();
 
-    int counter = 2 * track.sampleFreq / 3;
+    gint counter = 2 * track.sampleFreq / 3;
     while (isAlive())
     {
         if (getOffset() != -1)
@@ -820,10 +837,10 @@ static void* decodeStream(void* data)
 
         lockAcquire();
         short iPlaying = MpcPlugin.output->buffer_playing()? 1 : 0;
-        int iFree = MpcPlugin.output->buffer_free();
+        gint iFree = MpcPlugin.output->buffer_free();
         if (!mpcDecoder.isPause &&  iFree >= ((1152 * 4) << iPlaying))
         {
-            unsigned status = processBuffer(sampleBuffer, xmmsBuffer, decoder);
+            unsigned status = processBuffer(data, sampleBuffer, xmmsBuffer, decoder);
             if (status == (unsigned) (-1))
             {
                 mpcDecoder.isError = g_strdup_printf("[xmms-musepack] error from internal decoder on %s", filename);
@@ -839,7 +856,7 @@ static void* decodeStream(void* data)
                 counter -= status;
                 if(counter < 0)
                 {
-                    MpcPlugin.set_info(track.display, track.length, track.bitrate, track.sampleFreq, track.channels);
+                    data->set_params(data, track.display, track.length, track.bitrate, track.sampleFreq, track.channels);
                     counter = 2 * track.sampleFreq / 3;
                 }
             }
@@ -847,13 +864,14 @@ static void* decodeStream(void* data)
         else
         {
             lockRelease();
-            xmms_usleep(10000);
+            g_usleep(10000);
         }
     }
     return endThread(filename, input, false);
 }
 
-static int processBuffer(MPC_SAMPLE_FORMAT* sampleBuffer, char* xmmsBuffer, mpc_decoder& decoder)
+static int processBuffer(InputPlayback *playback, 
+    MPC_SAMPLE_FORMAT* sampleBuffer, char* xmmsBuffer, mpc_decoder& decoder)
 {
     mpc_uint32_t vbrAcc = 0;
     mpc_uint32_t vbrUpd = 0;
@@ -866,7 +884,7 @@ static int processBuffer(MPC_SAMPLE_FORMAT* sampleBuffer, char* xmmsBuffer, mpc_
         track.bitrate = static_cast<int> (vbrUpd * track.sampleFreq / 1152);
     }
 
-    produce_audio(MpcPlugin.output->written_time(), FMT_S16_LE, track.channels, status * 4, xmmsBuffer, NULL);
+    playback->pass_audio(playback, FMT_S16_LE, track.channels, status * 4, xmmsBuffer, NULL);
     return status;
 }
 
