@@ -1,15 +1,17 @@
 #ifndef XMMS_MUSEPACK
 #define XMMS_MUSEPACK
 
+#include "config.h"
+
 //xmms headers
 extern "C"
 {
-#include "audacious/plugin.h"
-#include "audacious/output.h"
-#include "audacious/util.h"
-#include "audacious/configdb.h"
-#include "audacious/titlestring.h"
-#include "audacious/vfs.h"
+#include <audacious/plugin.h>
+#include <audacious/output.h>
+#include <audacious/util.h>
+#include <audacious/configdb.h>
+#include <audacious/main.h>
+#include <audacious/i18n.h>
 }
 
 //stdlib headers
@@ -67,7 +69,7 @@ typedef struct Widgets
 
 typedef struct MpcDecoder
 {
-    char*      isError;
+    gchar*     isError;
     double     offset;
     bool       isOutput;
     bool       isAlive;
@@ -76,27 +78,25 @@ typedef struct MpcDecoder
 
 typedef struct TrackInfo
 {
-    int   bitrate;
-    char* display;
-    int   length;
-    int   sampleFreq;
-    int   channels;
+    gint   bitrate;
+    gchar* display;
+    gint   length;
+    gint   sampleFreq;
+    gint   channels;
 };
 
 typedef struct MpcInfo
 {
-    char*    title;
-    char*    artist;
-    char*    album;
-    char*    comment;
-    char*    genre;
-    char*    date;
-    unsigned track;
-    unsigned year;
+    gchar*    title;
+    gchar*    artist;
+    gchar*    album;
+    gchar*    comment;
+    gchar*    genre;
+    gchar*    date;
+    unsigned  track;
+    unsigned  year;
 };
 
-extern "C" InputPlugin * get_iplugin_info(void);
-
 static void       mpcOpenPlugin();
 static void       mpcAboutBox();
 static void       mpcConfigBox();
@@ -108,7 +108,7 @@ static void       mpcPlay(InputPlayback *data);
 static void       mpcStop(InputPlayback *data);
 static void       mpcPause(InputPlayback *data, short);
 static void       mpcSeek(InputPlayback *data, int);
-static void       mpcSetEq(int, float, float*);
+//static void       mpcSetEq(int, float, float*);
 static int        mpcGetTime(InputPlayback *data);
 static void       mpcGetSongInfo(char*, char**, int*);
 static void       freeTags(MpcInfo&);
@@ -125,16 +125,16 @@ static void       closeInfoBox(GtkWidget*, gpointer);
 static char*      mpcGenerateTitle(const MpcInfo&, char*);
 static void       lockAcquire();
 static void       lockRelease();
-static void*      decodeStream(void*);
-static int        processBuffer(MPC_SAMPLE_FORMAT*, char*, mpc_decoder&);
-static void*      endThread(char*, FILE*, bool);
+static void*      decodeStream(InputPlayback*);
+static int        processBuffer(InputPlayback*, MPC_SAMPLE_FORMAT*, char*, mpc_decoder&);
+//static void*      endThread(char*, FILE*, bool);
 static bool       isAlive();
 static void       setAlive(bool);
 static double     getOffset();
 static void       setOffset(double);
 static bool       isPause();
 static void       setReplaygain(mpc_streaminfo&, mpc_decoder&);
-static TitleInput* mpcGetSongTuple(char *);
+static Tuple *    mpcGetSongTuple(char *);
 
 #ifdef MPC_FIXED_POINT
 inline static int shiftSigned(MPC_SAMPLE_FORMAT val, int shift)
