@@ -40,106 +40,6 @@
 #include <audacious/plugin.h>
 #include <audacious/strings.h>
 
-static gchar *
-str_replace_char(gchar * str, gchar old, gchar new)
-{
-    gchar *match;
-
-    g_return_val_if_fail(str != NULL, NULL);
-
-    match = str;
-    while ((match = strchr(match, old)))
-        *match = new;
-
-    return str;
-}
-
-static gchar *
-str_replace_drive_letter(gchar * str)
-{
-    gchar *match, *match_end;
-
-    g_return_val_if_fail(str != NULL, NULL);
-
-    while ((match = strstr(str, ":\\"))) {
-        match--;
-        match_end = match + 3;
-        *match++ = '/';
-        while (*match_end)
-            *match++ = *match_end++;
-        *match = 0; /* the end of line */
-    }
-
-    return str;
-}
-
-static gchar *
-_convert_dos_path(gchar * path)
-{
-    g_return_val_if_fail(path != NULL, NULL);
-
-    /* replace drive letter with '/' */
-    str_replace_drive_letter(path);
-
-    /* replace '\' with '/' */
-    str_replace_char(path, '\\', '/');
-
-    return path;
-}
-
-static gchar *
-_construct_uri(gchar *string, const gchar *playlist_name) // uri, path and anything else
-{
-    gchar *filename = g_strdup(string);
-    gchar *tmp, *path, *tfile;
-    gchar *uri = NULL;
-
-    /* try to translate dos path */
-    _convert_dos_path(filename); /* in place replacement */
-
-    /* convert backslash to slash */
-    while ((tmp = strchr(filename, '\\')) != NULL)
-        *tmp = '/';
-
-    // make full path uri here
-    // case 1: filename is raw full path or uri
-    if (filename[0] == '/' || strstr(filename, "://")) {
-        uri = g_filename_to_uri(filename, NULL, NULL);
-        if(!uri) {
-            uri = g_strdup(filename);
-        }
-        g_free(filename);
-    }
-
-    // case 2: filename is not raw full path nor uri, playlist path is full path
-    // make full path by replacing last part of playlist path with filename. (using g_build_filename)
-    else if (playlist_name[0] == '/' || strstr(playlist_name, "://")) {
-        path = g_strdup(playlist_name);
-        tmp = strrchr(path, '/'); *tmp = '\0';
-        tmp = g_build_filename(path, filename, NULL);
-        g_free(path); g_free(filename);
-
-        // make sure we pass a 100% valid URI into the core.
-        tfile = g_filename_from_uri(tmp, NULL, NULL);
-        uri = g_filename_to_uri(tfile, NULL, NULL);
-        g_free(tfile);
-
-        if (uri)
-            g_free(tmp);
-        else
-            uri = tmp;
-    }
-
-    // case 3: filename is not raw full path nor uri, playlist path is not full path
-    // just abort.
-    else {
-        g_free(filename);
-        return NULL;
-    }
-
-    return uri;
-}
-
 static void
 playlist_load_pls(const gchar * filename, gint pos)
 {
@@ -147,13 +47,18 @@ playlist_load_pls(const gchar * filename, gint pos)
     gchar line_key[10], title_key[10];
     gchar *line, *title;
     Playlist *playlist = aud_playlist_get_active();
+    gchar *uri = NULL;
 
     g_return_if_fail(filename != NULL);
 
     if (!aud_str_has_suffix_nocase(filename, ".pls"))
         return;
 
-    INIFile *inifile = aud_open_ini_file(filename);
+    uri = g_filename_to_uri(filename, NULL, NULL);
+
+    INIFile *inifile = aud_open_ini_file(uri ? uri : filename);
+    g_free(uri); uri = NULL;
+
     if (!(line = aud_read_ini_string(inifile, "playlist", "NumberOfEntries")))
     {
         aud_close_ini_file(inifile);
@@ -167,7 +72,7 @@ playlist_load_pls(const gchar * filename, gint pos)
         g_snprintf(line_key, sizeof(line_key), "File%d", i);
         if ((line = aud_read_ini_string(inifile, "playlist", line_key)))
         {
-            gchar *uri = _construct_uri(line, filename);
+            gchar *uri = aud_construct_uri(line, filename);
             g_free(line);
 
             /* add file only if valid uri has been constructed */
@@ -204,6 +109,9 @@ playlist_save_pls(const gchar *filename, gint pos)
     VFSFile *file = aud_vfs_fopen(uri, "wb");
     Playlist *playlist = aud_playlist_get_active();
 
+    AUDDBG("filename=%s\n", filename);
+    AUDDBG("uri=%s\n", uri);
+
     g_return_if_fail(file != NULL);
     g_return_if_fail(playlist != NULL);
 
