@@ -1,5 +1,5 @@
-#ifndef _XS_LENGTH_H
-#define _XS_LENGTH_H
+#ifndef XS_LENGTH_H
+#define XS_LENGTH_H
 
 #include "xmms-sid.h"
 #include "xs_md5.h"
@@ -8,8 +8,7 @@
 extern "C" {
 #endif
 
-/*
- * Defines and typedefs
+/* Types
  */
 typedef struct _t_xs_sldb_node {
 	t_xs_md5hash	md5Hash;	/* 128-bit MD5 hash-digest */
@@ -22,24 +21,18 @@ typedef struct _t_xs_sldb_node {
 typedef struct {
 	t_xs_sldb_node	*pNodes,
 			**ppIndex;
-	gint		n;
+	size_t		n;
 } t_xs_sldb;
 
 
-/*
- * Functions
+/* Functions
  */
 gint			xs_sldb_read(t_xs_sldb *, const gchar *);
 gint			xs_sldb_index(t_xs_sldb *);
 void			xs_sldb_free(t_xs_sldb *);
 t_xs_sldb_node *	xs_sldb_get(t_xs_sldb *, const gchar *);
 
-
-gint			xs_songlen_init(void);
-void			xs_songlen_close(void);
-t_xs_sldb_node *	xs_songlen_get(const gchar *);
-
 #ifdef __cplusplus
 }
 #endif
-#endif /* _XS_LENGTH_H */
+#endif /* XS_LENGTH_H */
