@@ -2,6 +2,8 @@
  * Copyright (C) Tony Arcieri <bascule@inferno.tusculum.edu>
  * Copyright (C) 2001-2002  Haavard Kvaalen <havardk@xmms.org>
  * Copyright (C) 2007 William Pitcock <nenolod@sacredspiral.co.uk>
+ * Copyright (C) 2008 Cristi Măgherușan <majeru@gentoo.ro>
+ * Copyright (C) 2008 Eugene Zagidullin <e.asphyx@gmail.com>
  *
  * ReplayGain processing Copyright (C) 2002 Gian-Carlo Pascutto <gcp@sjeng.org>
  *
@@ -32,8 +34,8 @@
  */
 
 #include "config.h"
-
-#define DEBUG
+/*#define AUD_DEBUG
+#define DEBUG*/
 
 #define REMOVE_NONEXISTANT_TAG(x)   if (x != NULL && !*x) { x = NULL; }
 
@@ -72,9 +74,9 @@ static gchar *vorbis_generate_title(OggVorbis_File * vorbisfile, gchar * fn);
 static void vorbis_aboutbox(void);
 static void vorbis_init(void);
 static void vorbis_cleanup(void);
-static long vorbis_process_replaygain(float **pcm, int samples, int ch,
-                                      char *pcmout, float rg_scale);
-static gboolean vorbis_update_replaygain(float *scale);
+static long vorbis_interleave_buffer(float **pcm, int samples, int ch,
+                                     float *pcmout);
+static gboolean vorbis_update_replaygain(ReplayGainInfo *rg_info);
 
 static size_t ovcb_read(void *ptr, size_t size, size_t nmemb,
                         void *datasource);
@@ -108,10 +110,10 @@ InputPlugin vorbis_ip = {
     .pause = vorbis_pause,
     .seek = vorbis_seek,
     .cleanup = vorbis_cleanup,
-    .file_info_box = vorbis_file_info_box,       /* file info box, tag editing */
     .get_song_tuple = get_song_tuple,
     .is_our_file_from_vfs = vorbis_check_fd,
     .vfs_extensions = vorbis_fmts,
+    .update_song_tuple = vorbis_update_song_tuple,
 };
 
 InputPlugin *vorbis_iplist[] = { &vorbis_ip, NULL };
@@ -230,94 +232,8 @@ do_seek(InputPlayback *playback)
     }
 }
 
-static int
-vorbis_process_data(InputPlayback *playback, int last_section, 
-		    gboolean use_rg, float rg_scale)
-{
-    char pcmout[4096];
-    int bytes;
-    float **pcm;
-
-    /*
-     * A vorbis physical bitstream may consist of many logical
-     * sections (information for each of which may be fetched from
-     * the vf structure).  This value is filled in by ov_read to
-     * alert us what section we're currently decoding in case we
-     * need to change playback settings at a section boundary
-     */
-    int current_section = last_section;
-
-    g_mutex_lock(vf_mutex);
-    if (use_rg) {
-        bytes =
-            ov_read_float(&vf, &pcm, sizeof(pcmout) / 2 / channels,
-                          &current_section);
-        if (bytes > 0)
-            bytes = vorbis_process_replaygain(pcm, bytes, channels,
-                                              pcmout, rg_scale);
-    }
-    else {
-        bytes = ov_read(&vf, pcmout, sizeof(pcmout),
-                        (int) (G_BYTE_ORDER == G_BIG_ENDIAN),
-                        2, 1, &current_section);
-    }
-
-    /*
-     * We got some sort of error. Bail.
-     */
-    if (bytes <= 0 && bytes != OV_HOLE) {
-        g_mutex_unlock(vf_mutex);
-        playback->playing = 0;
-        playback->output->buffer_free();
-        playback->output->buffer_free();
-        playback->eof = TRUE;
-        return last_section;
-    }
-
-    if (current_section != last_section) {
-        /*
-         * The info struct is different in each section.  vf
-         * holds them all for the given bitstream.  This
-         * requests the current one
-         */
-        vorbis_info *vi = ov_info(&vf, -1);
-
-        if (vi->channels > 2) {
-            playback->eof = TRUE;
-            g_mutex_unlock(vf_mutex);
-            return current_section;
-        }
-
-
-        if (vi->rate != samplerate || vi->channels != channels) {
-            samplerate = vi->rate;
-            channels = vi->channels;
-            playback->output->buffer_free();
-            playback->output->buffer_free();
-            playback->output->close_audio();
-            if (!playback->output->
-                open_audio(FMT_S16_NE, vi->rate, vi->channels)) {
-                playback->error = TRUE;
-                playback->eof = TRUE;
-                g_mutex_unlock(vf_mutex);
-                return current_section;
-            }
-            playback->output->flush(ov_time_tell(&vf) * 1000);
-        }
-    }
-
-    g_mutex_unlock(vf_mutex);
-
-    playback->pass_audio(playback, FMT_S16_NE, channels, bytes, pcmout, &playback->playing);
-
-    if (!playback->playing)
-        return current_section;
-
-    if (seekneeded != -1)
-        do_seek(playback);
-
-    return current_section;
-}
+#define PCM_FRAMES 1024
+#define PCM_BUFSIZE PCM_FRAMES*2
 
 static gpointer
 vorbis_play_loop(gpointer arg)
@@ -336,8 +252,10 @@ vorbis_play_loop(gpointer arg)
     VFSFile *stream = NULL;
     void *datasource = NULL;
 
-    gboolean use_rg;
-    float rg_scale = 1.0;
+    /*gboolean use_rg;
+    /float rg_scale = 1.0;*/
+
+    ReplayGainInfo rg_info;
 
     memset(&vf, 0, sizeof(vf));
 
@@ -350,12 +268,24 @@ vorbis_play_loop(gpointer arg)
     fd->fd = stream;
     datasource = (void *) fd;
 
+    /*char pcmout[4096];*/
+    float pcmout[PCM_BUFSIZE*sizeof(float)];
+    int bytes;
+    float **pcm;
+
     /*
      * The open function performs full stream detection and
      * machine initialization.  None of the rest of ov_xx() works
      * without it
+     *
+     * A vorbis physical bitstream may consist of many logical
+     * sections (information for each of which may be fetched from
+     * the vf structure).  This value is filled in by ov_read to
+     * alert us what section we're currently decoding in case we
+     * need to change playback settings at a section boundary
      */
-
+ 
+   
     g_mutex_lock(vf_mutex);
     if (ov_open_callbacks(datasource, &vf, NULL, 0, aud_vfs_is_streaming(fd->fd) ? vorbis_callbacks_stream : vorbis_callbacks) < 0) {
         vorbis_callbacks.close_func(datasource);
@@ -377,8 +307,9 @@ vorbis_play_loop(gpointer arg)
     }
 
     title = vorbis_generate_title(&vf, filename);
-    use_rg = vorbis_update_replaygain(&rg_scale);
-
+    vorbis_update_replaygain(&rg_info);
+    playback->set_replaygain_info(playback, &rg_info);
+    
     vi = ov_info(&vf, -1);
 
     samplerate = vi->rate;
@@ -388,7 +319,7 @@ vorbis_play_loop(gpointer arg)
     g_mutex_unlock(vf_mutex);
 
     playback->set_params(playback, title, time, br, samplerate, channels);
-    if (!playback->output->open_audio(FMT_S16_NE, vi->rate, vi->channels)) {
+    if (!playback->output->open_audio(FMT_FLOAT, vi->rate, vi->channels)) {
         playback->error = TRUE;
         goto play_cleanup;
     }
@@ -403,47 +334,125 @@ vorbis_play_loop(gpointer arg)
      */
 
     while (playback->playing) {
-        int current_section;
-
-        if (seekneeded != -1)
-            do_seek(playback);
-
+       
         if (playback->eof) {
             g_usleep(20000);
             continue;
         }
 
-        current_section = vorbis_process_data(playback, last_section, 
-					      use_rg, rg_scale);
+        if (seekneeded != -1)
+            do_seek(playback);
 
-        if (current_section != last_section) {
+        
+        int current_section = last_section;
+
+        g_mutex_lock(vf_mutex);
+        
+        bytes = ov_read_float(&vf, &pcm, PCM_FRAMES, &current_section);
+        
+        if (bytes > 0)
+            bytes = vorbis_interleave_buffer(pcm, bytes, channels, pcmout);
+
+        /*
+         * We got some sort of error. Bail.
+         */
+        if (bytes <= 0 && bytes != OV_HOLE) {
+           /*
+            * EOF
+            */
+            AUDDBG("EOF\n");
+            playback->playing = 0;
+            playback->eof = TRUE;
+            current_section = last_section;
+        }
+
+        if (current_section <= last_section) {
             /*
-             * set total play time, bitrate, rate, and channels of
-             * current section
+             * The info struct is different in each section.  vf
+             * holds them all for the given bitstream.  This
+             * requests the current one
              */
+            vorbis_info *vi = ov_info(&vf, -1);
+
+            if (vi->channels > 2) {
+                playback->eof = TRUE;
+                g_mutex_unlock(vf_mutex);
+                goto stop_processing;
+            }
+
+
+            if (vi->rate != samplerate || vi->channels != channels) {
+                samplerate = vi->rate;
+                channels = vi->channels;
+                while(playback->output->buffer_playing()) g_usleep(50000);
+                playback->output->close_audio();
+                if (!playback->output->
+                        open_audio(FMT_FLOAT, vi->rate, vi->channels)) {
+                    playback->error = TRUE;
+                    playback->eof = TRUE;
+                    g_mutex_unlock(vf_mutex);
+                    goto stop_processing;
+                }
+                playback->output->flush(ov_time_tell(&vf) * 1000);
+                vorbis_update_replaygain(&rg_info);
+                playback->set_replaygain_info(playback, &rg_info); /* audio reopened */
+            }
+        }
+
+        g_mutex_unlock(vf_mutex);
+
+        playback->pass_audio(playback, FMT_FLOAT, channels, bytes, pcmout, &playback->playing);
+
+        if (!playback->playing)
+            goto stop_processing;
+
+        if (seekneeded != -1)
+            do_seek(playback);
+
+        stop_processing:
+   
+        if (current_section <= last_section) {
+            /*
+            * set total play time, bitrate, rate, and channels of
+            * current section
+            */
             if (title)
                 g_free(title);
 
             g_mutex_lock(vf_mutex);
             title = vorbis_generate_title(&vf, filename);
-            use_rg = vorbis_update_replaygain(&rg_scale);
 
             if (time != -1)
                 time = ov_time_total(&vf, -1) * 1000;
 
             g_mutex_unlock(vf_mutex);
-
+        
             playback->set_params(playback, title, time, br, samplerate, channels);
+
             timercount = playback->output->output_time();
 
             last_section = current_section;
+         
         }
-    }
-    if (!playback->error)
-        playback->output->close_audio();
-    /* fall through intentional */
+    } /* main loop */
 
-  play_cleanup:
+    if (!playback->error) {
+        /*this loop makes it not skip the last ~4 seconds, but the playback 
+         * timer isn't updated in this period, so it still needs a bit of work 
+         *
+         * majeru
+         */
+        if(playback->eof) /* do it only on EOF --asphyx */
+            while(playback->output->buffer_playing()) {
+                AUDDBG("waiting for empty output buffer\n");
+                g_usleep(50000);
+            }
+
+        playback->output->close_audio();
+    }
+
+
+    play_cleanup:
     g_free(title);
 
     /*
@@ -474,7 +483,9 @@ vorbis_stop(InputPlayback *playback)
 {
     if (playback->playing) {
         playback->playing = 0;
+        AUDDBG("waiting for playback thread finished\n");
         g_thread_join(thread);
+        AUDDBG("playback finished\n");
     }
 }
 
@@ -494,110 +505,43 @@ vorbis_seek(InputPlayback *data, int time)
 }
 
 /* Make sure you've locked vf_mutex */
+
 static gboolean
-vorbis_update_replaygain(float *scale)
+vorbis_update_replaygain(ReplayGainInfo *rg_info)
 {
     vorbis_comment *comment;
-    char *rg_gain = NULL, *rg_peak_str = NULL;
-    float rg_peak;
+    char *rg_gain = NULL, *rg_peak = NULL;
 
-    if (!vorbis_cfg.use_replaygain && !vorbis_cfg.use_anticlip)
-        return FALSE;
-    if ((comment = ov_comment(&vf, -1)) == NULL)
+    if (rg_info == NULL || (comment = ov_comment(&vf, -1)) == NULL)
         return FALSE;
 
-    *scale = 1.0;
+    rg_gain = vorbis_comment_query(comment, "replaygain_album_gain", 0);
+    if (!rg_gain) rg_gain = vorbis_comment_query(comment, "rg_audiophile", 0);    /* Old */
+    rg_info->album_gain = rg_gain != NULL ? atof(rg_gain) : 0.0;
+    
+    rg_gain = vorbis_comment_query(comment, "replaygain_track_gain", 0);
+    if (!rg_gain) rg_gain = vorbis_comment_query(comment, "rg_radio", 0);    /* Old */
+    rg_info->track_gain = rg_gain != NULL ? atof(rg_gain) : 0.0;
+    
+    rg_peak = vorbis_comment_query(comment, "replaygain_album_peak", 0);
+    rg_info->album_peak = rg_peak != NULL ? atof(rg_peak) : 0.0;
+    
+    rg_peak = vorbis_comment_query(comment, "replaygain_track_peak", 0);
+    if (!rg_peak) rg_peak = vorbis_comment_query(comment, "rg_peak", 0);  /* Old */
+    rg_info->track_peak = rg_peak != NULL ? atof(rg_peak) : 0.0;
 
-    if (vorbis_cfg.use_replaygain) {
-        if (vorbis_cfg.replaygain_mode == REPLAYGAIN_MODE_ALBUM) {
-            rg_gain =
-                vorbis_comment_query(comment, "replaygain_album_gain", 0);
-            if (!rg_gain)
-                rg_gain = vorbis_comment_query(comment, "rg_audiophile", 0);    /* Old */
-        }
-
-        if (!rg_gain)
-            rg_gain =
-                vorbis_comment_query(comment, "replaygain_track_gain", 0);
-        if (!rg_gain)
-            rg_gain = vorbis_comment_query(comment, "rg_radio", 0); /* Old */
-
-        /* FIXME: Make sure this string is the correct format first? */
-        if (rg_gain)
-            *scale = pow(10., atof(rg_gain) / 20);
-    }
-
-    if (vorbis_cfg.use_anticlip) {
-        if (vorbis_cfg.replaygain_mode == REPLAYGAIN_MODE_ALBUM)
-            rg_peak_str =
-                vorbis_comment_query(comment, "replaygain_album_peak", 0);
-
-        if (!rg_peak_str)
-            rg_peak_str =
-                vorbis_comment_query(comment, "replaygain_track_peak", 0);
-        if (!rg_peak_str)
-            rg_peak_str = vorbis_comment_query(comment, "rg_peak", 0);  /* Old */
-
-        if (rg_peak_str)
-            rg_peak = atof(rg_peak_str);
-        else
-            rg_peak = 1;
-
-        if (*scale * rg_peak > 1.0)
-            *scale = 1.0 / rg_peak;
-    }
-
-    if (*scale != 1.0 || vorbis_cfg.use_booster) {
-        /* safety */
-        if (*scale > 15.0)
-            *scale = 15.0;
-
-        return TRUE;
-    }
-
-    return FALSE;
+    return TRUE;
 }
 
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-#  define GET_BYTE1(val) ((val) >> 8)
-#  define GET_BYTE2(val) ((val) & 0xff)
-#else
-#  define GET_BYTE1(val) ((val) & 0xff)
-#  define GET_BYTE2(val) ((val) >> 8)
-#endif
-
 static long
-vorbis_process_replaygain(float **pcm, int samples, int ch,
-                          char *pcmout, float rg_scale)
+vorbis_interleave_buffer(float **pcm, int samples, int ch, float *pcmout)
 {
     int i, j;
-    /* ReplayGain processing */
     for (i = 0; i < samples; i++)
-        for (j = 0; j < ch; j++) {
-            float sample = pcm[j][i] * rg_scale;
-            int value;
+        for (j = 0; j < ch; j++)
+            *pcmout++ = pcm[j][i];
 
-            if (vorbis_cfg.use_booster) {
-                sample *= 2;
-
-                /* hard 6dB limiting */
-                if (sample < -0.5)
-                    sample = tanh((sample + 0.5) / 0.5) * 0.5 - 0.5;
-                else if (sample > 0.5)
-                    sample = tanh((sample - 0.5) / 0.5) * 0.5 + 0.5;
-            }
-
-            value = sample * 32767;
-            if (value > 32767)
-                value = 32767;
-            else if (value < -32767)
-                value = -32767;
-
-            *pcmout++ = GET_BYTE1(value);
-            *pcmout++ = GET_BYTE2(value);
-        }
-
-    return 2 * ch * samples;
+    return ch * samples * sizeof(float);
 }
 
 static void _aud_tuple_associate_string(Tuple *tuple, const gint nfield, const gchar *field, const gchar *string)
@@ -618,7 +562,7 @@ get_aud_tuple_for_vorbisfile(OggVorbis_File * vorbisfile, gchar *filename)
     VFSVorbisFile *vfd = (VFSVorbisFile *) vorbisfile->datasource;
     Tuple *tuple = NULL;
     gint length;
-    vorbis_comment *comment;
+    vorbis_comment *comment = NULL;
 
     tuple = aud_tuple_new_from_filename(filename);
 
@@ -629,6 +573,8 @@ get_aud_tuple_for_vorbisfile(OggVorbis_File * vorbisfile, gchar *filename)
 
     /* associate with tuple */
     aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, length);
+    /* maybe, it would be better to display nominal bitrate (like in main win), not average? --eugene */
+    aud_tuple_associate_int(tuple, FIELD_BITRATE, NULL, ov_bitrate(vorbisfile, -1)/1000);
 
     if ((comment = ov_comment(vorbisfile, -1))) {
         gchar *tmps;
@@ -641,19 +587,21 @@ get_aud_tuple_for_vorbisfile(OggVorbis_File * vorbisfile, gchar *filename)
 
         if ((tmps = vorbis_comment_query(comment, "tracknumber", 0)) != NULL)
             aud_tuple_associate_int(tuple, FIELD_TRACK_NUMBER, NULL, atoi(tmps));
-
-        aud_tuple_associate_string(tuple, FIELD_QUALITY, NULL, "lossy");
-
-        if (comment && comment->vendor)
-        {
-            gchar *codec = g_strdup_printf("Ogg Vorbis [%s]", comment->vendor);
-            aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, codec);
-            g_free(codec);
-        }
-        else
-            aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, "Ogg Vorbis");
     }
 
+    aud_tuple_associate_string(tuple, FIELD_QUALITY, NULL, "lossy");
+
+    if (comment && comment->vendor)
+    {
+        gchar *codec = g_strdup_printf("Ogg Vorbis [%s]", comment->vendor);
+        aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, codec);
+        g_free(codec);
+    }
+    else
+        aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, "Ogg Vorbis");
+    
+    aud_tuple_associate_string(tuple, FIELD_MIMETYPE, NULL, "application/ogg");
+
     return tuple;
 }
 
@@ -746,7 +694,8 @@ vorbis_aboutbox(void)
                                         "Jack Moffitt <jack@icecast.org>\n"
                                         "Jorn Baayen <jorn@nl.linux.org>\n"
                                         "Haavard Kvaalen <havardk@xmms.org>\n"
-                                        "Gian-Carlo Pascutto <gcp@sjeng.org>\n\n"
+                                        "Gian-Carlo Pascutto <gcp@sjeng.org>\n"
+                                        "Eugene Zagidullin <e.asphyx@gmail.com>\n\n"
                                         "Visit the Xiph.org Foundation at http://www.xiph.org/\n"),
                                        _("Ok"), FALSE, NULL, NULL);
       g_signal_connect(G_OBJECT(about_window), "destroy",
@@ -770,10 +719,6 @@ vorbis_init(void)
     vorbis_cfg.proxy_pass = NULL;
     vorbis_cfg.tag_override = FALSE;
     vorbis_cfg.tag_format = NULL;
-    vorbis_cfg.use_anticlip = FALSE;
-    vorbis_cfg.use_replaygain = FALSE;
-    vorbis_cfg.replaygain_mode = REPLAYGAIN_MODE_TRACK;
-    vorbis_cfg.use_booster = FALSE;
 
     db = aud_cfg_db_open();
     aud_cfg_db_get_int(db, "vorbis", "http_buffer_size",
@@ -791,13 +736,6 @@ vorbis_init(void)
     if (!aud_cfg_db_get_string(db, "vorbis", "tag_format",
                                &vorbis_cfg.tag_format))
         vorbis_cfg.tag_format = g_strdup("%p - %t");
-    aud_cfg_db_get_bool(db, "vorbis", "use_anticlip",
-                        &vorbis_cfg.use_anticlip);
-    aud_cfg_db_get_bool(db, "vorbis", "use_replaygain",
-                        &vorbis_cfg.use_replaygain);
-    aud_cfg_db_get_int(db, "vorbis", "replaygain_mode",
-                       &vorbis_cfg.replaygain_mode);
-    aud_cfg_db_get_bool(db, "vorbis", "use_booster", &vorbis_cfg.use_booster);
 
     aud_cfg_db_get_bool(db, NULL, "use_proxy", &vorbis_cfg.use_proxy);
     aud_cfg_db_get_string(db, NULL, "proxy_host", &vorbis_cfg.proxy_host);
@@ -880,7 +818,7 @@ ovcb_close(void *datasource)
     if (handle->probe == FALSE)
     {
         ret = aud_vfs_fclose(handle->fd);
-//        g_free(handle); // it causes double free. i'm not really sure that commenting out at here is correct. --yaz
+/*        g_free(handle);  it causes double free. i'm not really sure that commenting out at here is correct. --yaz*/
     }
 
     return ret;
